@@ -155,7 +155,7 @@ bus_dbus_impl_destroy (BusDBusImpl *dbus)
     }
     g_list_free (dbus->rules);
     dbus->rules = NULL;
-    
+
     for (p = dbus->connections; p != NULL; p = p->next) {
         BusConnection *connection = BUS_CONNECTION (p->data);
         g_signal_handlers_disconnect_by_func (connection, _connection_destroy_cb, dbus);
@@ -205,6 +205,7 @@ _dbus_introspect (BusDBusImpl     *dbus,
         "      <arg direction=\"in\" type=\"s\"/>\n"
         "      <arg direction=\"out\" type=\"u\"/>\n"
         "    </method>\n"
+#if 0
         "    <method name=\"StartServiceByName\">\n"
         "      <arg direction=\"in\" type=\"s\"/>\n"
         "      <arg direction=\"in\" type=\"u\"/>\n"
@@ -213,6 +214,7 @@ _dbus_introspect (BusDBusImpl     *dbus,
         "    <method name=\"UpdateActivationEnvironment\">\n"
         "      <arg direction=\"in\" type=\"a{ss}\"/>\n"
         "    </method>\n"
+#endif
         "    <method name=\"NameHasOwner\">\n"
         "      <arg direction=\"in\" type=\"s\"/>\n"
         "      <arg direction=\"out\" type=\"b\"/>\n"
@@ -220,9 +222,11 @@ _dbus_introspect (BusDBusImpl     *dbus,
         "    <method name=\"ListNames\">\n"
         "      <arg direction=\"out\" type=\"as\"/>\n"
         "    </method>\n"
+#if 0
         "    <method name=\"ListActivatableNames\">\n"
         "      <arg direction=\"out\" type=\"as\"/>\n"
         "    </method>\n"
+#endif
         "    <method name=\"AddMatch\">\n"
         "      <arg direction=\"in\" type=\"s\"/>\n"
         "    </method>\n"
@@ -233,6 +237,7 @@ _dbus_introspect (BusDBusImpl     *dbus,
         "      <arg direction=\"in\" type=\"s\"/>\n"
         "      <arg direction=\"out\" type=\"s\"/>\n"
         "    </method>\n"
+#if 0
         "    <method name=\"ListQueuedOwners\">\n"
         "      <arg direction=\"in\" type=\"s\"/>\n"
         "      <arg direction=\"out\" type=\"as\"/>\n"
@@ -255,6 +260,7 @@ _dbus_introspect (BusDBusImpl     *dbus,
         "    </method>\n"
         "    <method name=\"ReloadConfig\">\n"
         "    </method>\n"
+#endif
         "    <method name=\"GetId\">\n"
         "      <arg direction=\"out\" type=\"s\"/>\n"
         "    </method>\n"
@@ -263,12 +269,14 @@ _dbus_introspect (BusDBusImpl     *dbus,
         "      <arg type=\"s\"/>\n"
         "      <arg type=\"s\"/>\n"
         "    </signal>\n"
+#if 0
         "    <signal name=\"NameLost\">\n"
         "      <arg type=\"s\"/>\n"
         "    </signal>\n"
         "    <signal name=\"NameAcquired\">\n"
         "      <arg type=\"s\"/>\n"
         "    </signal>\n"
+#endif
         "  </interface>\n"
         "</node>\n";
 
@@ -349,23 +357,30 @@ _dbus_list_names (BusDBusImpl       *dbus,
     IBusMessage *reply_message;
     IBusMessageIter iter, sub_iter;
     GList *name, *names;
+    gchar *v;
 
     reply_message = ibus_message_new_method_return (message);
 
-    ibus_message_iter_init_append (message, &iter);
+    ibus_message_iter_init_append (reply_message, &iter);
     ibus_message_iter_open_container (&iter, IBUS_TYPE_ARRAY, "s", &sub_iter);
 
-    // append unique names
-    names = g_hash_table_get_keys (dbus->unique_names);
+    v = DBUS_SERVICE_DBUS;
+    ibus_message_iter_append (&sub_iter, G_TYPE_STRING, &v);
 
+    v = IBUS_SERVICE_IBUS;
+    ibus_message_iter_append (&sub_iter, G_TYPE_STRING, &v);
+
+    // append well-known names
+    names = g_hash_table_get_keys (dbus->names);
     names = g_list_sort (names, (GCompareFunc) g_strcmp0);
     for (name = names; name != NULL; name = name->next) {
         ibus_message_iter_append (&sub_iter, G_TYPE_STRING, &(name->data));
     }
     g_list_free (names);
 
-    // append well-known names
-    names = g_hash_table_get_keys (dbus->names);
+    // append unique names
+    names = g_hash_table_get_keys (dbus->unique_names);
+
     names = g_list_sort (names, (GCompareFunc) g_strcmp0);
     for (name = names; name != NULL; name = name->next) {
         ibus_message_iter_append (&sub_iter, G_TYPE_STRING, &(name->data));
@@ -626,7 +641,9 @@ _dbus_request_name (BusDBusImpl     *dbus,
         return reply_message;
     }
 
-    if (g_hash_table_lookup (dbus->names, name) != NULL) {
+    if (g_strcmp0 (name, DBUS_SERVICE_DBUS) == 0 ||
+        g_strcmp0 (name, IBUS_SERVICE_IBUS) == 0 ||
+        g_hash_table_lookup (dbus->names, name) != NULL) {
         reply_message = ibus_message_new_error_printf (message,
                                                        DBUS_ERROR_FAILED,
                                                        "Name %s has owner",
@@ -808,7 +825,7 @@ _connection_ibus_message_cb (BusConnection  *connection,
     if (G_UNLIKELY (IBUS_OBJECT_DESTROYED (dbus))) {
         return FALSE;
     }
-    
+
     if (ibus_message_is_signal (message,
                                 DBUS_INTERFACE_LOCAL,
                                 "Disconnected")) {
@@ -1071,12 +1088,6 @@ bus_dbus_impl_dispatch_message_by_rule (BusDBusImpl     *dbus,
 
     dbus_message_set_data (message, data_slot, (gpointer) TRUE, NULL);
 
-#if 0
-    if (g_strcmp0 (ibus_message_get_member (message), "ValueChanged") == 0) {
-        g_debug ("Dispatch ValueChanged");
-    }
-#endif
-
     for (link = dbus->rules; link != NULL; link = link->next) {
         GList *list = bus_match_rule_get_recipients (BUS_MATCH_RULE (link->data),
                                                      message);
@@ -1109,7 +1120,7 @@ bus_dbus_impl_register_object (BusDBusImpl *dbus,
     g_assert (IBUS_IS_SERVICE (object));
 
     const gchar *path;
-    
+
     if (G_UNLIKELY (IBUS_OBJECT_DESTROYED (dbus))) {
         return FALSE;
     }
@@ -1136,7 +1147,7 @@ bus_dbus_impl_unregister_object (BusDBusImpl *dbus,
     g_assert (IBUS_IS_SERVICE (object));
 
     const gchar *path;
-    
+
     if (G_UNLIKELY (IBUS_OBJECT_DESTROYED (dbus))) {
         return FALSE;
     }
