@@ -20,10 +20,10 @@
 
 #include <ibusinternal.h>
 #include <ibusmarshalers.h>
+#include "ibusimpl.h"
 #include "engineproxy.h"
 
-#define BUS_ENGINE_PROXY_GET_PRIVATE(o)  \
-   (G_TYPE_INSTANCE_GET_PRIVATE ((o), BUS_TYPE_ENGINE_PROXY, BusEngineProxyPrivate))
+#define PROCESS_KEY_EVENT_TIMEOUT (10000)
 
 enum {
     COMMIT_TEXT,
@@ -47,15 +47,6 @@ enum {
 };
 
 
-/* BusEngineProxyPriv */
-struct _BusEngineProxyPrivate {
-    gboolean enabled;
-    IBusEngineDesc *desc;
-
-    IBusPropList *prop_list;
-};
-typedef struct _BusEngineProxyPrivate BusEngineProxyPrivate;
-
 static guint    engine_signals[LAST_SIGNAL] = { 0 };
 // static guint            engine_signals[LAST_SIGNAL] = { 0 };
 
@@ -112,10 +103,15 @@ bus_engine_proxy_new (const gchar    *path,
                                               "connection", connection,
                                               NULL);
 
-    BusEngineProxyPrivate *priv;
-    priv = BUS_ENGINE_PROXY_GET_PRIVATE (engine);
-    priv->desc = desc;
+    engine->desc = desc;
     g_object_ref (desc);
+    if (desc->layout != NULL && desc->layout[0] != '\0') {
+        engine->keymap = ibus_keymap_new (desc->layout);
+    }
+
+    if (engine->keymap == NULL) {
+        engine->keymap = ibus_keymap_new ("us");
+    }
 
     return engine;
 }
@@ -129,8 +125,6 @@ bus_engine_proxy_class_init (BusEngineProxyClass *klass)
 
     parent_class = (IBusProxyClass *) g_type_class_peek_parent (klass);
 
-    g_type_class_add_private (klass, sizeof (BusEngineProxyPrivate));
-
     ibus_object_class->destroy = (IBusObjectDestroyFunc) bus_engine_proxy_real_destroy;
 
     proxy_class->ibus_signal = bus_engine_proxy_ibus_signal;
@@ -153,9 +147,10 @@ bus_engine_proxy_class_init (BusEngineProxyClass *klass)
             G_SIGNAL_RUN_LAST,
             0,
             NULL, NULL,
-            ibus_marshal_VOID__UINT_UINT,
+            ibus_marshal_VOID__UINT_UINT_UINT,
             G_TYPE_NONE,
-            2,
+            3,
+            G_TYPE_UINT,
             G_TYPE_UINT,
             G_TYPE_UINT);
 
@@ -323,23 +318,18 @@ bus_engine_proxy_class_init (BusEngineProxyClass *klass)
 static void
 bus_engine_proxy_init (BusEngineProxy *engine)
 {
-    BusEngineProxyPrivate *priv;
-    priv = BUS_ENGINE_PROXY_GET_PRIVATE (engine);
-
-    priv->enabled = FALSE;
-    priv->prop_list = NULL;
-    priv->desc = NULL;
+    engine->enabled = FALSE;
+    engine->prop_list = NULL;
+    engine->desc = NULL;
+    engine->keymap = NULL;
 }
 
 static void
 bus_engine_proxy_real_destroy (BusEngineProxy *engine)
 {
-    BusEngineProxyPrivate *priv;
-    priv = BUS_ENGINE_PROXY_GET_PRIVATE (engine);
-
-    if (priv->prop_list) {
-        g_object_unref (priv->prop_list);
-        priv->prop_list = NULL;
+    if (engine->prop_list) {
+        g_object_unref (engine->prop_list);
+        engine->prop_list = NULL;
     }
 
     if (ibus_proxy_get_connection ((IBusProxy *) engine)) {
@@ -348,9 +338,14 @@ bus_engine_proxy_real_destroy (BusEngineProxy *engine)
                          G_TYPE_INVALID);
     }
 
-    if (priv->desc) {
-        g_object_unref (priv->desc);
-        priv->desc = NULL;
+    if (engine->desc) {
+        g_object_unref (engine->desc);
+        engine->desc = NULL;
+    }
+
+    if (engine->keymap) {
+        g_object_unref (engine->keymap);
+        engine->keymap = NULL;
     }
 
     IBUS_OBJECT_CLASS(parent_class)->destroy (IBUS_OBJECT (engine));
@@ -364,7 +359,6 @@ bus_engine_proxy_ibus_signal (IBusProxy     *proxy,
     g_assert (message != NULL);
 
     BusEngineProxy *engine;
-    BusEngineProxyPrivate *priv;
     IBusError *error;
     gint i;
 
@@ -386,7 +380,6 @@ bus_engine_proxy_ibus_signal (IBusProxy     *proxy,
     };
 
     engine = BUS_ENGINE_PROXY (proxy);
-    priv = BUS_ENGINE_PROXY_GET_PRIVATE (engine);
 
     for (i = 0; ; i++) {
         if (signals[i].member == NULL)
@@ -411,19 +404,25 @@ bus_engine_proxy_ibus_signal (IBusProxy     *proxy,
         g_object_unref (text);
     }
     else if (ibus_message_is_signal (message, IBUS_INTERFACE_ENGINE, "ForwardKeyEvent")) {
-        guint keyval;
-        guint states;
+        guint32 keyval;
+        guint32 keycode;
+        guint32 states;
         gboolean retval;
 
         retval = ibus_message_get_args (message,
                                         &error,
                                         G_TYPE_UINT, &keyval,
+                                        G_TYPE_UINT, &keycode,
                                         G_TYPE_UINT, &states,
                                         G_TYPE_INVALID);
 
         if (!retval)
             goto failed;
-        g_signal_emit (engine, engine_signals[FORWARD_KEY_EVENT], keyval, states);
+        g_signal_emit (engine,
+                       engine_signals[FORWARD_KEY_EVENT],
+                       keyval,
+                       keycode,
+                       states);
     }
     else if (ibus_message_is_signal (message, IBUS_INTERFACE_ENGINE, "UpdatePreeditText")) {
         IBusText *text;
@@ -482,20 +481,20 @@ bus_engine_proxy_ibus_signal (IBusProxy     *proxy,
     else if (ibus_message_is_signal (message, IBUS_INTERFACE_ENGINE, "RegisterProperties")) {
         gboolean retval;
 
-        if (priv->prop_list) {
-            g_object_unref (priv->prop_list);
-            priv->prop_list = NULL;
+        if (engine->prop_list) {
+            g_object_unref (engine->prop_list);
+            engine->prop_list = NULL;
         }
 
         retval = ibus_message_get_args (message,
                                         &error,
-                                        IBUS_TYPE_PROP_LIST, &priv->prop_list,
+                                        IBUS_TYPE_PROP_LIST, &engine->prop_list,
                                         G_TYPE_INVALID);
         if (!retval) {
-            priv->prop_list = NULL;
+            engine->prop_list = NULL;
             goto failed;
         }
-        g_signal_emit (engine, engine_signals[REGISTER_PROPERTIES], 0, priv->prop_list);
+        g_signal_emit (engine, engine_signals[REGISTER_PROPERTIES], 0, engine->prop_list);
     }
     else if (ibus_message_is_signal (message, IBUS_INTERFACE_ENGINE, "UpdateProperty")) {
         IBusProperty *prop;
@@ -537,20 +536,17 @@ bus_engine_proxy_process_key_event_reply_cb (IBusPendingCall *pending,
 {
     IBusMessage *reply_message;
     IBusError *error;
-    gboolean retval;
+    gboolean retval = FALSE;
 
     reply_message = dbus_pending_call_steal_reply (pending);
 
     if (reply_message == NULL) {
-        call_data->func(FALSE, call_data->user_data);
-        return;
+        goto _out;
     }
     else if ((error = ibus_error_new_from_message (reply_message)) != NULL) {
         g_warning ("%s: %s", error->name, error->message);
-        ibus_message_unref (reply_message);
         ibus_error_free (error);
-        call_data->func(FALSE, call_data->user_data);
-        return;
+        goto _out;
     }
 
     if (!ibus_message_get_args (reply_message,
@@ -558,12 +554,14 @@ bus_engine_proxy_process_key_event_reply_cb (IBusPendingCall *pending,
                                 G_TYPE_BOOLEAN, &retval,
                                 G_TYPE_INVALID)) {
         g_warning ("%s: %s", error->name, error->message);
-        ibus_message_unref (reply_message);
         ibus_error_free (error);
-        call_data->func (GINT_TO_POINTER (FALSE), call_data->user_data);
-        return;
+        goto _out;
     }
 
+_out:
+    if (reply_message) {
+        ibus_message_unref (reply_message);
+    }
     call_data->func (GINT_TO_POINTER (retval), call_data->user_data);
     g_slice_free (CallData, call_data);
 }
@@ -571,6 +569,7 @@ bus_engine_proxy_process_key_event_reply_cb (IBusPendingCall *pending,
 void
 bus_engine_proxy_process_key_event (BusEngineProxy *engine,
                                     guint           keyval,
+                                    guint           keycode,
                                     guint           state,
                                     GFunc           return_cb,
                                     gpointer        user_data)
@@ -583,12 +582,20 @@ bus_engine_proxy_process_key_event (BusEngineProxy *engine,
     IBusError *error;
     gboolean retval;
 
+    if (keycode != 0 && !BUS_DEFAULT_IBUS->use_sys_layout && engine->keymap != NULL) {
+        guint t = ibus_keymap_lookup_keysym (engine->keymap, keycode, state);
+        if (t != IBUS_VoidSymbol) {
+            keyval = t;
+        }
+    }
+
     retval = ibus_proxy_call_with_reply ((IBusProxy *) engine,
                                          "ProcessKeyEvent",
                                          &pending,
-                                         -1,
+                                         PROCESS_KEY_EVENT_TIMEOUT,
                                          &error,
                                          G_TYPE_UINT, &keyval,
+                                         G_TYPE_UINT, &keycode,
                                          G_TYPE_UINT, &state,
                                          G_TYPE_INVALID);
     if (!retval) {
@@ -609,6 +616,7 @@ bus_engine_proxy_process_key_event (BusEngineProxy *engine,
     ibus_pending_call_unref (pending);
 
     if (!retval) {
+        g_slice_free (CallData, call_data);
         g_warning ("%s : ProcessKeyEvent", DBUS_ERROR_NO_MEMORY);
         return_cb (GINT_TO_POINTER (FALSE), user_data);
         return;
@@ -729,8 +737,5 @@ bus_engine_proxy_get_desc (BusEngineProxy *engine)
 {
     g_assert (BUS_IS_ENGINE_PROXY (engine));
 
-    BusEngineProxyPrivate *priv;
-    priv = BUS_ENGINE_PROXY_GET_PRIVATE (engine);
-
-    return priv->desc;
+    return engine->desc;
 }
