@@ -59,8 +59,11 @@ static void     bus_ibus_impl_set_trigger       (BusIBusImpl        *ibus,
 static void     bus_ibus_impl_set_preload_engines
                                                 (BusIBusImpl        *ibus,
                                                  GValue             *value);
-static void     _factory_destroy_cb             (BusFactoryProxy      *factory,
-                                                 BusIBusImpl          *ibus);
+static void     bus_ibus_impl_set_use_sys_layout
+                                                (BusIBusImpl        *ibus,
+                                                 GValue             *value);
+static void     _factory_destroy_cb             (BusFactoryProxy    *factory,
+                                                 BusIBusImpl        *ibus);
 
 static IBusServiceClass  *parent_class = NULL;
 
@@ -240,6 +243,15 @@ bus_ibus_impl_set_preload_engines (BusIBusImpl *ibus,
     }
 }
 
+static void
+bus_ibus_impl_set_use_sys_layout (BusIBusImpl *ibus,
+                                  GValue      *value)
+{
+    if (value != NULL && G_VALUE_TYPE (value) == G_TYPE_BOOLEAN) {
+        ibus->use_sys_layout = g_value_get_boolean (value);
+    }
+}
+
 static gint
 _engine_desc_cmp (IBusEngineDesc *desc1,
                   IBusEngineDesc *desc2)
@@ -320,6 +332,7 @@ bus_ibus_impl_reload_config (BusIBusImpl *ibus)
         { "general/hotkey", "next_engine", bus_ibus_impl_set_next_engine },
         { "general/hotkey", "prev_engine", bus_ibus_impl_set_prev_engine },
         { "general", "preload_engines", bus_ibus_impl_set_preload_engines },
+        { "general", "use_system_keyboard_layout", bus_ibus_impl_set_use_sys_layout },
         { NULL, NULL, NULL },
     };
 
@@ -362,6 +375,7 @@ _config_value_changed_cb (IBusConfig  *config,
         { "general/hotkey", "next_engine", bus_ibus_impl_set_next_engine },
         { "general/hotkey", "prev_engine", bus_ibus_impl_set_prev_engine },
         { "general", "preload_engines",    bus_ibus_impl_set_preload_engines },
+        { "general", "use_system_keyboard_layout", bus_ibus_impl_set_use_sys_layout },
         { NULL, NULL, NULL },
     };
 
@@ -485,6 +499,9 @@ bus_ibus_impl_init (BusIBusImpl *ibus)
     ibus->config = NULL;
 
     ibus->hotkey_profile = ibus_hotkey_profile_new ();
+    ibus->keymap = ibus_keymap_new ("us");
+
+    ibus->use_sys_layout = FALSE;
 
     bus_ibus_impl_reload_config (ibus);
 
@@ -550,6 +567,11 @@ bus_ibus_impl_destroy (BusIBusImpl *ibus)
         ibus->hotkey_profile = NULL;
     }
 
+    if (ibus->keymap != NULL) {
+        g_object_unref (ibus->keymap);
+        ibus->keymap = NULL;
+    }
+
     bus_server_quit (BUS_DEFAULT_SERVER);
     ibus_object_destroy ((IBusObject *) BUS_DEFAULT_SERVER);
     IBUS_OBJECT_CLASS(parent_class)->destroy (IBUS_OBJECT (ibus));
@@ -569,17 +591,30 @@ _ibus_introspect (BusIBusImpl     *ibus,
         "      <arg name=\"data\" direction=\"out\" type=\"s\"/>\n"
         "    </method>\n"
         "  </interface>\n"
-        "  <interface name=\"org.freedesktop.DBus\">\n"
-        "    <method name=\"RequestName\">\n"
-        "      <arg direction=\"in\" type=\"s\"/>\n"
-        "      <arg direction=\"in\" type=\"u\"/>\n"
-        "      <arg direction=\"out\" type=\"u\"/>\n"
+        "  <interface name=\"org.freedesktop.IBus\">\n"
+        "    <method name=\"GetAddress\">\n"
+        "      <arg name=\"address\" direction=\"out\" type=\"s\"/>\n"
+        "    </method>\n"
+        "    <method name=\"CreateInputContext\">\n"
+        "      <arg name=\"name\" direction=\"in\" type=\"s\"/>\n"
+        "      <arg name=\"context\" direction=\"out\" type=\"o\"/>\n"
+        "    </method>\n"
+        "    <method name=\"RegisterComponent\">\n"
+        "      <arg name=\"components\" direction=\"in\" type=\"v\"/>\n"
+        "    </method>\n"
+        "    <method name=\"ListEngines\">\n"
+        "      <arg name=\"engines\" direction=\"out\" type=\"av\"/>\n"
+        "    </method>\n"
+        "    <method name=\"ListActiveEngines\">\n"
+        "      <arg name=\"engines\" direction=\"out\" type=\"av\"/>\n"
+        "    </method>\n"
+        "    <method name=\"Exit\">\n"
+        "      <arg name=\"restart\" direction=\"in\" type=\"b\"/>\n"
+        "    </method>\n"
+        "    <method name=\"Ping\">\n"
+        "      <arg name=\"data\" direction=\"in\" type=\"v\"/>\n"
+        "      <arg name=\"data\" direction=\"out\" type=\"v\"/>\n"
         "    </method>\n"
-        "    <signal name=\"NameOwnerChanged\">\n"
-        "      <arg type=\"s\"/>\n"
-        "      <arg type=\"s\"/>\n"
-        "      <arg type=\"s\"/>\n"
-        "    </signal>\n"
         "  </interface>\n"
         "</node>\n";
 
@@ -1070,10 +1105,10 @@ _ibus_exit (BusIBusImpl     *ibus,
         gint fd;
 
         exe = g_strdup_printf ("/proc/%d/exe", getpid ());
-        if (!g_file_test (exe, G_FILE_TEST_EXISTS)) {
-            g_free (exe);
-            exe = g_argv[0];
-        }
+        exe = g_file_read_link (exe, NULL);
+
+        if (exe == NULL)
+            exe = BINDIR"/ibus-daemon";
 
         /* close all fds except stdin, stdout, stderr */
         for (fd = 3; fd <= sysconf (_SC_OPEN_MAX); fd ++) {
@@ -1091,6 +1126,24 @@ _ibus_exit (BusIBusImpl     *ibus,
     return NULL;
 }
 
+static IBusMessage *
+_ibus_ping (BusIBusImpl     *ibus,
+            IBusMessage     *message,
+            BusConnection   *connection)
+{
+    IBusMessage *reply;
+    IBusMessageIter src, dst;
+
+    reply = ibus_message_new_method_return (message);
+
+    ibus_message_iter_init (message, &src);
+    ibus_message_iter_init_append (reply, &dst);
+
+    ibus_message_iter_copy_data (&dst, &src);
+
+    return reply;
+}
+
 static gboolean
 bus_ibus_impl_ibus_message (BusIBusImpl     *ibus,
                             BusConnection   *connection,
@@ -1110,7 +1163,7 @@ bus_ibus_impl_ibus_message (BusIBusImpl     *ibus,
     } handlers[] =  {
         /* Introspectable interface */
         { DBUS_INTERFACE_INTROSPECTABLE,
-                               "Introspect", _ibus_introspect },
+                               "Introspect",            _ibus_introspect },
         /* IBus interface */
         { IBUS_INTERFACE_IBUS, "GetAddress",            _ibus_get_address },
         { IBUS_INTERFACE_IBUS, "CreateInputContext",    _ibus_create_input_context },
@@ -1118,30 +1171,33 @@ bus_ibus_impl_ibus_message (BusIBusImpl     *ibus,
         { IBUS_INTERFACE_IBUS, "ListEngines",           _ibus_list_engines },
         { IBUS_INTERFACE_IBUS, "ListActiveEngines",     _ibus_list_active_engines },
         { IBUS_INTERFACE_IBUS, "Exit",                  _ibus_exit },
+        { IBUS_INTERFACE_IBUS, "Ping",                  _ibus_ping },
         { NULL, NULL, NULL }
     };
 
     ibus_message_set_sender (message, bus_connection_get_unique_name (connection));
     ibus_message_set_destination (message, DBUS_SERVICE_DBUS);
 
-    for (i = 0; handlers[i].interface != NULL; i++) {
-        if (ibus_message_is_method_call (message,
-                                         handlers[i].interface,
-                                         handlers[i].name)) {
+    if (ibus_message_get_type (message) == DBUS_MESSAGE_TYPE_METHOD_CALL) {
+        for (i = 0; handlers[i].interface != NULL; i++) {
+            if (ibus_message_is_method_call (message,
+                                             handlers[i].interface,
+                                             handlers[i].name)) {
 
-            reply_message = handlers[i].handler (ibus, message, connection);
-            if (reply_message) {
+                reply_message = handlers[i].handler (ibus, message, connection);
+                if (reply_message) {
 
-                ibus_message_set_sender (reply_message, DBUS_SERVICE_DBUS);
-                ibus_message_set_destination (reply_message, bus_connection_get_unique_name (connection));
-                ibus_message_set_no_reply (reply_message, TRUE);
+                    ibus_message_set_sender (reply_message, DBUS_SERVICE_DBUS);
+                    ibus_message_set_destination (reply_message, bus_connection_get_unique_name (connection));
+                    ibus_message_set_no_reply (reply_message, TRUE);
 
-                ibus_connection_send ((IBusConnection *) connection, reply_message);
-                ibus_message_unref (reply_message);
+                    ibus_connection_send ((IBusConnection *) connection, reply_message);
+                    ibus_message_unref (reply_message);
+                }
+
+                g_signal_stop_emission_by_name (ibus, "ibus-message");
+                return TRUE;
             }
-
-            g_signal_stop_emission_by_name (ibus, "ibus-message");
-            return TRUE;
         }
     }
 
@@ -1171,6 +1227,15 @@ bus_ibus_impl_get_hotkey_profile (BusIBusImpl *ibus)
     return ibus->hotkey_profile;
 }
 
+IBusKeymap *
+bus_ibus_impl_get_keymap (BusIBusImpl *ibus)
+{
+
+    g_assert (BUS_IS_IBUS_IMPL (ibus));
+
+    return ibus->keymap;
+}
+
 BusRegistry *
 bus_ibus_impl_get_registry (BusIBusImpl *ibus)
 {
