@@ -23,6 +23,7 @@
 #include <sys/socket.h>
 #include <sys/time.h>
 #include <sys/un.h>
+#include <string.h>
 #include <ibus.h>
 #include "ibusimcontext.h"
 
@@ -216,11 +217,13 @@ _key_snooper_cb (GtkWidget   *widget,
     case GDK_KEY_RELEASE:
         retval = ibus_input_context_process_key_event (ibusimcontext->ibuscontext,
                                                        event->keyval,
+                                                       event->hardware_keycode - 8,
                                                        event->state | IBUS_RELEASE_MASK);
         break;
     case GDK_KEY_PRESS:
         retval = ibus_input_context_process_key_event (ibusimcontext->ibuscontext,
                                                        event->keyval,
+                                                       event->hardware_keycode - 8,
                                                        event->state);
         break;
     default:
@@ -419,11 +422,13 @@ ibus_im_context_filter_keypress (GtkIMContext *context,
         case GDK_KEY_RELEASE:
             retval = ibus_input_context_process_key_event (ibusimcontext->ibuscontext,
                                                            event->keyval,
+                                                           event->hardware_keycode - 8,
                                                            event->state | IBUS_RELEASE_MASK);
             break;
         case GDK_KEY_PRESS:
             retval = ibus_input_context_process_key_event (ibusimcontext->ibuscontext,
                                                            event->keyval,
+                                                           event->hardware_keycode - 8,
                                                            event->state);
             break;
         default:
@@ -769,14 +774,22 @@ _ibus_context_update_preedit_text_cb (IBusInputContext  *ibuscontext,
     ibusimcontext->preedit_visible = visible;
 
     if (ibusimcontext->preedit_visible) {
-        if (flag)
+        if (flag) {
+            /* invisible => visible */
             g_signal_emit (ibusimcontext, _signal_preedit_start_id, 0);
+        }
         g_signal_emit (ibusimcontext, _signal_preedit_changed_id, 0);
     }
     else {
-        g_signal_emit (ibusimcontext, _signal_preedit_changed_id, 0);
-        if (flag)
+        if (flag) {
+            /* visible => invisible */
+            g_signal_emit (ibusimcontext, _signal_preedit_changed_id, 0);
             g_signal_emit (ibusimcontext, _signal_preedit_end_id, 0);
+        }
+        else {
+            /* still invisible */
+            /* do nothing */
+        }
     }
 }
 
