@@ -24,7 +24,7 @@ import dbus
 
 class Application:
     def __init__ (self):
-        self._dbusconn = dbus.connection.Connection (ibus.IBUS_ADDR)
+        self._dbusconn = dbus.connection.Connection (ibus.get_address())
         self._dbusconn.add_signal_receiver (self._disconnected_cb,
                             "Disconnected",
                             dbus_interface = dbus.LOCAL_IFACE)
