@@ -27,6 +27,7 @@ __all__ = (
         "ATTR_UNDERLINE_SINGLE",
         "ATTR_UNDERLINE_DOUBLE",
         "ATTR_UNDERLINE_LOW",
+        "ATTR_UNDERLINE_ERROR",
         "Attribute",
         "AttributeUnderline",
         "AttributeForeground",
@@ -47,8 +48,10 @@ ATTR_UNDERLINE_NONE = 0
 ATTR_UNDERLINE_SINGLE = 1
 ATTR_UNDERLINE_DOUBLE = 2
 ATTR_UNDERLINE_LOW = 3
+ATTR_UNDERLINE_ERROR = 4
 
 class Attribute(Serializable):
+    __gtype_name__ = "PYIBusAttribute"
     __NAME__ = "IBusAttribute"
     def __init__ (self, type=0, value=0, start_index=0, end_index=0):
         super(Attribute, self).__init__()
@@ -91,8 +94,6 @@ class Attribute(Serializable):
         self.__start_index = struct.pop(0)
         self.__end_index = struct.pop(0)
 
-serializable_register(Attribute)
-
 class AttributeUnderline (Attribute):
     def __init__(self, value, start_index, end_index):
         Attribute.__init__ (self, ATTR_TYPE_UNDERLINE, value, start_index, end_index)
@@ -112,6 +113,7 @@ def RGB (r, g, b):
     return ARGB (255, r, g, b)
 
 class AttrList(Serializable):
+    __gtype_name__ = "PYIBusAttrList"
     __NAME__ = "IBusAttrList"
     def __init__ (self, attrs = []):
         super(AttrList, self).__init__()
@@ -137,8 +139,6 @@ class AttrList(Serializable):
     def __iter__ (self):
         return self._attrs.__iter__ ()
 
-serializable_register(AttrList)
-
 def test():
     attr_list = AttrList()
     attr_list.append (Attribute())
