@@ -36,6 +36,7 @@ import config
 dbus.mainloop.glib.DBusGMainLoop(set_as_default = True)
 
 class Bus(object.Object):
+    __gtype_name__ = "PYIBusBus"
     __gsignals__ = {
         "disconnected" : (
             gobject.SIGNAL_RUN_LAST,
@@ -51,7 +52,7 @@ class Bus(object.Object):
 
     def __init__(self):
         super(Bus, self).__init__()
-        self.__dbusconn = dbus.connection.Connection(common.IBUS_ADDR)
+        self.__dbusconn = dbus.connection.Connection(common.get_address())
         self.__dbus = self.__dbusconn.get_object(dbus.BUS_DAEMON_NAME,
                                                  dbus.BUS_DAEMON_PATH)
         self.__unique_name = self.hello()
@@ -107,7 +108,7 @@ class Bus(object.Object):
         return self.__dbusconn
 
     def get_address(self):
-        return common.IBUS_ADDR
+        return common.get_address()
 
     # define ibus methods
     def register_component(self, component):
@@ -128,6 +129,21 @@ class Bus(object.Object):
     def exit(self, restart):
         return self.__ibus.Exit(restart)
 
+    def ping(self, data):
+        flag = isinstance(data, serializable.Serializable)
+        if flag:
+            data = serializable.serialize_object(data)
+        data = self.__ibus.Ping(data, dbus_interface="org.freedesktop.IBus")
+        if flag:
+            data = serializable.deserialize_object(data)
+        return data
+
+    def introspect_ibus(self):
+        return self.__ibus.Introspect()
+
+    def introspect_dbus(self):
+        return self.__dbus.Introspect()
+
     def get_config(self):
         try:
             return self.__config
