@@ -74,6 +74,7 @@ class EngineFactoryProxy(interface.IEngineFactory):
         self.remove_from_connection ()
 
 class FactoryInfo(Serializable):
+    __gtype_name__ = "PYIBusFactoryInfo"
     __NAME__ = "IBusFactoryInfo"
     def __init__ (self, path=None, name=None, lang=None, icon=None, authors=None, credits=None):
         super(FactoryInfo, self).__init__()
@@ -129,5 +130,3 @@ class FactoryInfo(Serializable):
         self.__authors = struct.pop(0)
         self.__credits = struct.pop(0)
 
-serializable_register(FactoryInfo)
-
