@@ -32,6 +32,7 @@ import common
 import serializable
 
 class InputContext(object.Object):
+    __gtype_name__ = "PYIBusInputContext"
     __gsignals__ = {
         "commit-text" : (
             gobject.SIGNAL_RUN_LAST,
@@ -115,13 +116,16 @@ class InputContext(object.Object):
         ),
     }
 
-    def __init__(self, bus, path):
+    def __init__(self, bus, path, watch_signals=False):
         super(InputContext, self).__init__()
 
         self.__bus = bus
         self.__context = bus.get_dbusconn().get_object(common.IBUS_SERVICE_IBUS, path)
         self.__signal_matches = []
 
+        if not watch_signals:
+            return
+
         m = self.__context.connect_to_signal("CommitText", self.__commit_text_cb)
         self.__signal_matches.append(m)
         m = self.__context.connect_to_signal("UpdatePreeditText", self.__update_preedit_text_cb)
@@ -169,17 +173,18 @@ class InputContext(object.Object):
     def __update_auxiliary_text_cb(self, *args):
         text = serializable.deserialize_object(args[0])
         visible = args[1]
-        self.emit("update-auxiliray-text", text, visible)
+        self.emit("update-auxiliary-text", text, visible)
 
     def __update_lookup_table_cb(self, *args):
         table = serializable.deserialize_object(args[0])
         visible = args[1]
         self.emit("update-lookup-table", table, visible)
 
-    def process_key_event(self, keyval, modifiers):
+    def process_key_event(self, keyval, keycode, modifiers):
         keyval = dbus.UInt32(keyval)
+        keycode = dbus.UInt32(keycode)
         modifiers = dbus.UInt32(modifiers)
-        return self.__context.ProcessKeyEvent(keyval, modifiers)
+        return self.__context.ProcessKeyEvent(keyval, keycode, modifiers)
 
     def set_cursor_location(self, x, y, w, h):
         x = dbus.Int32(x)
@@ -210,7 +215,13 @@ class InputContext(object.Object):
         caps = dbus.UInt32(caps)
         return self.__context.SetCapabilities(caps)
 
+    def detach_signals(self):
+        for m in self.__signal_matches:
+            m.remove()
+        del self.__signal_matches[:]
+
     def destroy(self):
+        self.detach_signals()
         self.__context.Destroy()
         super(InputContext, self).destroy()
 
@@ -225,6 +236,10 @@ class InputContext(object.Object):
     def set_engine(self, engine):
         return self.__context.SetEngine(engine.name)
 
+    def introspect(self):
+        return self.__context.Introspect()
+
+
 
 def test():
     import gtk
