@@ -46,16 +46,6 @@ class IIBus(dbus.service.Object):
     @method(in_signature="s", out_signature="o")
     def CreateInputContext(self, client_name, dbusconn): pass
 
-    @method(in_signature="s", out_signature="sb")
-    def GetInputContextStates(self, ic, dbusconn): pass
-
-    # methods for ibus engine provide
-    @method(in_signature="av")
-    def RegisterFactories(self, object_paths, dbusconn): pass
-
-    @method(in_signature="av")
-    def UnregisterFactories(self, object_paths, dbusconn): pass
-
     # general methods
     @method(in_signature="av")
     def RegisterComponent(self, components, dbusconn): pass
@@ -69,3 +59,6 @@ class IIBus(dbus.service.Object):
     @method(in_signature="b")
     def Exit(self, restart, dbusconn): pass
 
+    @method(in_signature="v", out_signature="v")
+    def Ping(self, data, dbusconn): pass
+
