@@ -28,6 +28,7 @@ from serializable import *
 from exception import *
 
 class LookupTable(Serializable):
+    __gtype_name__ = "PYIBusLookupTable"
     __NAME__ = "IBusLookupTable"
     def __init__(self, page_size=5, cursor_pos=0, coursor_visible=True, round=False, candidates=None, labels=None):
         super(LookupTable, self).__init__()
@@ -214,9 +215,6 @@ class LookupTable(Serializable):
         self.__candidates = map(deserialize_object, struct.pop(0))
         self.__labels = map(deserialize_object, struct.pop(0))
 
-
-serializable_register(LookupTable)
-
 def test():
     t = LookupTable()
     # attrs = AttrList()
