@@ -33,11 +33,15 @@ BUTTON2_MASK = 1 << 9
 BUTTON3_MASK = 1 << 10
 BUTTON4_MASK = 1 << 11
 BUTTON5_MASK = 1 << 12
-  
+
+HANDLED_MASK = 1 << 24
+IGNORED_MASK = 1 << 25
+FORWARD_MASK = 1 << 25
+
 SUPER_MASK = 1 << 26
 HYPER_MASK = 1 << 27
 META_MASK = 1 << 28
-  
+
 RELEASE_MASK = 1 << 30
 
 MODIFIER_MASK = 0x5c001fff
