@@ -59,6 +59,7 @@ def _to_text(text):
     return Text(text)
 
 class Property(Serializable):
+    __gtype_name__ = "PYIBusProperty"
     __NAME__ = "IBusProperty"
     def __init__(self, key="", type=PROP_TYPE_NORMAL, label=u"", icon=u"", tooltip=u"",
                  sensitive=True, visible=True, state=PROP_STATE_UNCHECKED):
@@ -173,9 +174,8 @@ class Property(Serializable):
 
         self.__sub_props = deserialize_object(props)
 
-serializable_register(Property)
-
 class PropList(Serializable):
+    __gtype_name__ = "PYIBusPropList"
     __NAME__ = "IBusPropList"
     def __init__(self):
         super(PropList, self).__init__()
@@ -218,8 +218,6 @@ class PropList(Serializable):
     def __getitem__(self, i):
         return self.__props.__getitem__(i)
 
-serializable_register(PropList)
-
 def test():
     props = PropList()
     props.append(Property(u"a"))
