@@ -27,6 +27,10 @@ import ibus
 
 from icon import load_icon
 
+from gettext import dgettext
+_  = lambda a : dgettext("ibus", a)
+N_ = lambda a : a
+
 class EngineTreeView(gtk.TreeView):
     __gsignals__ = {
         'changed' : (
@@ -39,19 +43,21 @@ class EngineTreeView(gtk.TreeView):
         super(EngineTreeView, self).__init__()
 
         self.__engines = set([])
-
         self.__changed = False
 
-        self.__model = gtk.ListStore(gobject.TYPE_PYOBJECT)
+        # self.set_headers_visible(True)
+        self.set_reorderable(True)
+
+        self.__model = gtk.ListStore(gobject.TYPE_PYOBJECT, gobject.TYPE_STRING)
         self.set_model(self.__model)
         self.__model.connect("row-changed", self.__emit_changed_delay_cb, "row-changed")
         self.__model.connect("row-deleted", self.__emit_changed_delay_cb, "row-deleted")
         self.__model.connect("row-inserted", self.__emit_changed_delay_cb, "row-inserted")
         self.__model.connect("rows-reordered", self.__emit_changed_delay_cb, "rows-reordered")
 
-        self.set_headers_visible(False)
-
-        column = gtk.TreeViewColumn()
+        # create im name & icon column
+        column = gtk.TreeViewColumn(_("Input Method"))
+        column.set_min_width(220)
 
         renderer = gtk.CellRendererPixbuf()
         renderer.set_property("xalign", 0)
@@ -60,11 +66,31 @@ class EngineTreeView(gtk.TreeView):
 
         renderer = gtk.CellRendererText()
         renderer.set_property("xalign", 0)
+        renderer.set_property("ellipsize", pango.ELLIPSIZE_END)
         column.pack_start(renderer, True)
         column.set_cell_data_func(renderer, self.__name_cell_data_cb)
-        self.append_column (column)
+        self.append_column(column)
 
-        self.set_reorderable(True)
+        # create im keyboard layout column
+        renderer = gtk.CellRendererCombo()
+        model = gtk.ListStore(gobject.TYPE_STRING)
+        model.append(("us",))
+        model.append(("jp",))
+        model.append(("xkb",))
+        renderer.set_property("xalign", 0)
+        renderer.set_property("model", model)
+        renderer.set_property("text-column", 0)
+        renderer.set_property("has-entry", False)
+        renderer.set_property("editable", True)
+        renderer.connect("changed", self.__engine_layout_changed_cb)
+
+        column = gtk.TreeViewColumn(_("Kbd"))
+        column.set_expand(False)
+        column.set_fixed_width(32)
+        column.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
+        column.pack_start(renderer, False)
+        column.set_cell_data_func(renderer, self.__layout_cell_data_cb)
+        # self.append_column(column)
 
         self.set_engines(engines)
 
@@ -97,12 +123,30 @@ class EngineTreeView(gtk.TreeView):
         renderer.set_property("sensitive", True)
         language = ibus.get_language_name(engine.language)
         renderer.set_property("text", "%s - %s" % (language, engine.longname))
+        if self.__model.get_path(iter)[0] == 0:
+            # default engine
+            renderer.set_property("weight", pango.WEIGHT_BOLD)
+        else:
+            renderer.set_property("weight", pango.WEIGHT_NORMAL)
+
+    def __layout_cell_data_cb(self, celllayout, renderer, model, iter):
+        engine = self.__model.get_value(iter, 0)
+        layout = self.__model.get_value(iter, 1)
+        renderer.set_property("sensitive", True)
+        if not layout:
+            layout = engine.layout
+        renderer.set_property("text", layout)
         if self.__model.get_path(iter)[0] == 0:
             #default engine
             renderer.set_property("weight", pango.WEIGHT_BOLD)
         else:
             renderer.set_property("weight", pango.WEIGHT_NORMAL)
 
+    def __engine_layout_changed_cb(self, combo, path, iter):
+        return
+        i = self.__model.get_iter(path)
+        layout = combo.get_property("model").get_value(iter, 0)
+        self.__model.set_value(i, 1, layout)
 
     def set_engines(self, engines):
         self.__model.clear()
