@@ -144,7 +144,10 @@ class KeyboardShortcutSelection(gtk.VBox):
             return
         if shortcut in self.get_shortcuts():
             return
-        model.insert(-1, (shortcut,))
+        iter = model.insert(-1, (shortcut,))
+        self.__add_button.set_sensitive(False)
+        path = model.get_path(iter)
+        self.__shortcut_view.set_cursor(path)
 
     def __get_shortcut_from_buttons(self):
         modifiers = []
@@ -182,11 +185,13 @@ class KeyboardShortcutSelection(gtk.VBox):
         model = self.__shortcut_view.get_model()
         path, column = self.__shortcut_view.get_cursor()
         model[path[0]][0] = shortcut
+        self.__update_add_and_apply_buttons()
 
     def __del_selected_shortcut(self):
         model = self.__shortcut_view.get_model()
         path, column = self.__shortcut_view.get_cursor()
         del model[path[0]]
+        self.__update_add_and_apply_buttons()
 
     def __shortcut_view_cursor_changed_cb(self, treeview):
         shortcut = self.__get_selected_shortcut()
