@@ -19,7 +19,6 @@
 # Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 # Boston, MA  02111-1307  USA
 
-import locale
 import gettext
 import os
 import signal
@@ -69,7 +68,10 @@ class Setup(object):
 
     def __init__(self):
         super(Setup, self).__init__()
-        locale.bind_textdomain_codeset("ibus", "UTF-8")
+        localedir = os.getenv("IBUS_LOCALEDIR")
+        gettext.bindtextdomain("ibus", localedir)
+        glade.bindtextdomain("ibus", localedir)
+        gettext.bind_textdomain_codeset("ibus", "UTF-8")
         glade.textdomain("ibus")
         glade_file = path.join(path.dirname(__file__), "./setup.glade")
         self.__xml = glade.XML(glade_file)
@@ -99,7 +101,9 @@ class Setup(object):
 
         button = self.__xml.get_widget("button_trigger")
         entry = self.__xml.get_widget("entry_trigger")
-        entry.set_text("; ".join(shortcuts))
+        text = "; ".join(shortcuts)
+        entry.set_text(text)
+        entry.set_tooltip_text(text)
         button.connect("clicked", self.__shortcut_button_clicked_cb,
                     N_("trigger"), "general/hotkey", "trigger", entry)
 
@@ -109,7 +113,9 @@ class Setup(object):
                         ibus.CONFIG_GENERAL_SHORTCUT_NEXT_ENGINE_DEFAULT)
         button = self.__xml.get_widget("button_next_engine")
         entry = self.__xml.get_widget("entry_next_engine")
-        entry.set_text("; ".join(shortcuts))
+        text = "; ".join(shortcuts)
+        entry.set_text(text)
+        entry.set_tooltip_text(text)
         button.connect("clicked", self.__shortcut_button_clicked_cb,
                     N_("next input method"), "general/hotkey", "next_engine", entry)
 
@@ -119,7 +125,9 @@ class Setup(object):
                         ibus.CONFIG_GENERAL_SHORTCUT_PREV_ENGINE_DEFAULT)
         button = self.__xml.get_widget("button_prev_engine")
         entry = self.__xml.get_widget("entry_prev_engine")
-        entry.set_text("; ".join(shortcuts))
+        text = "; ".join(shortcuts)
+        entry.set_text(text)
+        entry.set_tooltip_text(text)
         button.connect("clicked", self.__shortcut_button_clicked_cb,
                     N_("previous input method"), "general/hotkey", "prev_engine", entry)
 
@@ -162,6 +170,12 @@ class Setup(object):
             self.__config.get_value("panel", "show_im_name", False))
         self.__checkbutton_show_im_name.connect("toggled", self.__checkbutton_show_im_name_toggled_cb)
 
+        # use system keyboard layout setting
+        self.__checkbutton_use_sys_layout = self.__xml.get_widget("checkbutton_use_sys_layout")
+        self.__checkbutton_use_sys_layout.set_active(
+            self.__config.get_value("general", "use_system_keyboard_layout", False))
+        self.__checkbutton_use_sys_layout.connect("toggled", self.__checkbutton_use_sys_layout_toggled_cb)
+
         # init engine page
         self.__engines = self.__bus.list_engines()
         self.__combobox = EngineComboBox(self.__engines)
@@ -198,9 +212,10 @@ class Setup(object):
 
     def __button_engine_about_cb(self, button):
         engine = self.__treeview.get_select_engine()
-        about = EngineAbout(engine)
-        about.run()
-        about.destroy()
+        if engine:
+            about = EngineAbout(engine)
+            about.run()
+            about.destroy()
 
     def __treeview_changed_cb(self, treeview):
         engines = self.__treeview.get_engines()
@@ -261,7 +276,9 @@ class Setup(object):
         if id != gtk.RESPONSE_OK:
             return
         self.__config.set_list(section, _name, shortcuts, "s")
-        entry.set_text("; ".join(shortcuts))
+        text = "; ".join(shortcuts)
+        entry.set_text(text)
+        entry.set_tooltip_text(text)
 
 
     def __item_started_column_toggled_cb(self, cell, path_str, model):
@@ -375,6 +392,10 @@ class Setup(object):
         value = self.__checkbutton_show_im_name.get_active()
         self.__config.set_value("panel", "show_im_name", value)
 
+    def __checkbutton_use_sys_layout_toggled_cb(self, button):
+        value = self.__checkbutton_use_sys_layout.get_active()
+        self.__config.set_value("general", "use_system_keyboard_layout", value)
+
     def __config_value_changed_cb(self, bus, section, name, value):
         pass
 
