@@ -20,6 +20,7 @@
 #ifndef __IBUS_H_
 #define __IBUS_H_
 
+#include <ibusversion.h>
 #include <ibusmainloop.h>
 #include <ibusshare.h>
 #include <ibusobject.h>
@@ -36,6 +37,7 @@
 #include <ibuslookuptable.h>
 #include <ibusbus.h>
 #include <ibuskeysyms.h>
+#include <ibuskeymap.h>
 #include <ibusmessage.h>
 #include <ibuspendingcall.h>
 #include <ibuserror.h>
