@@ -112,7 +112,7 @@ ibus_bus_class_init (IBusBusClass *klass)
      *
      * Emitted when IBusBus is connected.
      *
-     * <note><para>@user_data is not actually a valid parameter. It is displayed because of GtkDoc bug.</para></note>
+     * <note><para>Argument @user_data is ignored in this function.</para></note>
      */
     bus_signals[CONNECTED] =
         g_signal_new (I_("connected"),
@@ -129,7 +129,7 @@ ibus_bus_class_init (IBusBusClass *klass)
      *
      * Emitted when IBusBus is disconnected.
      *
-     * <note><para>@user_data is not actually a valid parameter. It is displayed because of GtkDoc bug.</para></note>
+     * <note><para>Argument @user_data is ignored in this function.</para></note>
      */
     bus_signals[DISCONNECTED] =
         g_signal_new (I_("disconnected"),
@@ -180,12 +180,10 @@ _connection_destroy_cb (IBusConnection  *connection,
 static void
 ibus_bus_connect (IBusBus *bus)
 {
-    const gchar *socket_path;
-    struct stat buf;
-
     IBusBusPrivate *priv;
     priv = IBUS_BUS_GET_PRIVATE (bus);
 
+#if 0
     socket_path = ibus_get_socket_path ();
 
     if (stat (socket_path, &buf) != 0) {
@@ -200,8 +198,10 @@ ibus_bus_connect (IBusBus *bus)
     if (priv->connection != NULL) {
         ibus_object_destroy ((IBusObject *) priv->connection);
     }
-
-    priv->connection = ibus_connection_open (ibus_get_address ());
+#endif
+    if (ibus_get_address () != NULL) {
+        priv->connection = ibus_connection_open (ibus_get_address ());
+    }
 
     if (priv->connection) {
         ibus_bus_hello (bus);
@@ -224,16 +224,11 @@ _changed_cb (GFileMonitor       *monitor,
              GFileMonitorEvent   event_type,
              IBusBus            *bus)
 {
-    static GFile *socket_file = NULL;
-
-    if (socket_file == NULL) {
-        socket_file = g_file_new_for_path (ibus_get_socket_path ());
-    }
-
-    if (event_type == G_FILE_MONITOR_EVENT_CREATED) {
-        if (g_file_equal (file, socket_file)) {
-            ibus_bus_connect (bus);
-        }
+    // g_debug ("changed %x", event_type);
+    if (ibus_bus_is_connected (bus))
+        return;
+    if (event_type == G_FILE_MONITOR_EVENT_CHANGED) {
+        ibus_bus_connect (bus);
     }
 }
 
@@ -241,7 +236,7 @@ static void
 ibus_bus_init (IBusBus *bus)
 {
     struct stat buf;
-    const gchar *path;
+    gchar *path;
     GFile *file;
 
     IBusBusPrivate *priv;
@@ -251,7 +246,7 @@ ibus_bus_init (IBusBus *bus)
     priv->connection = NULL;
     priv->watch_dbus_signal = FALSE;
 
-    path = ibus_get_socket_folder ();
+    path = g_path_get_dirname (ibus_get_socket_path ());
 
     if (stat (path, &buf) == 0) {
         if (buf.st_uid != ibus_get_daemon_uid ()) {
@@ -259,22 +254,25 @@ ibus_bus_init (IBusBus *bus)
             return;
         }
     }
+#if 0
     else {
         if (getuid () == ibus_get_daemon_uid ()) {
             mkdir (path, 0700);
             chmod (path, 0700);
         }
     }
+#endif
 
     ibus_bus_connect (bus);
 
 
-    file = g_file_new_for_path (path);
-    priv->monitor = g_file_monitor_directory (file, 0, NULL, NULL);
+    file = g_file_new_for_path (ibus_get_socket_path ());
+    priv->monitor = g_file_monitor_file (file, 0, NULL, NULL);
 
     g_signal_connect (priv->monitor, "changed", (GCallback) _changed_cb, bus);
 
     g_object_unref (file);
+    g_free (path);
 }
 
 static void
