@@ -80,6 +80,7 @@ static gboolean ibus_engine_ibus_message    (IBusEngine         *engine,
 static gboolean ibus_engine_process_key_event
                                             (IBusEngine         *engine,
                                              guint               keyval,
+                                             guint               keycode,
                                              guint               state);
 static void     ibus_engine_focus_in        (IBusEngine         *engine);
 static void     ibus_engine_focus_out       (IBusEngine         *engine);
@@ -227,13 +228,20 @@ ibus_engine_class_init (IBusEngineClass *klass)
     /**
      * IBusEngine::process-key-event:
      * @engine: An IBusEngine.
-     * @keyval: KeySym of the key press.
-     * @state: Key modifier flags
+     * @keyval: Key symbol of the key press.
+     * @keycode: KeyCode of the key press.
+     * @state: Key modifier flags.
      *
      * Emitted when a key event is received.
      * Implement the member function process_key_event() in extended class to receive this signal.
+     * Both the key symbol and keycode are passed to the member function.
+     * See ibus_input_context_process_key_event() for further explanation of
+     * key symbol, keycode and which to use.
      *
-     * <note><para>@user_data is not actually a valid parameter. It is displayed because of GtkDoc bug.</para></note>
+     * Returns: TRUE for successfully process the key; FALSE otherwise.
+     * See also:  ibus_input_context_process_key_event().
+     *
+     * <note><para>Argument @user_data is ignored in this function.</para></note>
      */
     engine_signals[PROCESS_KEY_EVENT] =
         g_signal_new (I_("process-key-event"),
@@ -241,9 +249,10 @@ ibus_engine_class_init (IBusEngineClass *klass)
             G_SIGNAL_RUN_LAST,
             G_STRUCT_OFFSET (IBusEngineClass, process_key_event),
             NULL, NULL,
-            ibus_marshal_BOOL__UINT_UINT,
+            ibus_marshal_BOOL__UINT_UINT_UINT,
             G_TYPE_BOOLEAN,
-            2,
+            3,
+            G_TYPE_UINT,
             G_TYPE_UINT,
             G_TYPE_UINT);
 
@@ -254,7 +263,8 @@ ibus_engine_class_init (IBusEngineClass *klass)
      * Emitted when the client application get the focus.
      * Implement the member function focus_in() in extended class to receive this signal.
      *
-     * <note><para>@user_data is not actually a valid parameter. It is displayed because of GtkDoc bug.</para></note>
+     * See also: ibus_input_context_focus_in()
+     * <note><para>Argument @user_data is ignored in this function.</para></note>
      */
     engine_signals[FOCUS_IN] =
         g_signal_new (I_("focus-in"),
@@ -273,7 +283,8 @@ ibus_engine_class_init (IBusEngineClass *klass)
      * Emitted when the client application  lost the focus.
      * Implement the member function focus_out() in extended class to receive this signal.
      *
-     * <note><para>@user_data is not actually a valid parameter. It is displayed because of GtkDoc bug.</para></note>
+     * See also: ibus_input_context_focus_out()
+     * <note><para>Argument @user_data is ignored in this function.</para></note>
      */
     engine_signals[FOCUS_OUT] =
         g_signal_new (I_("focus-out"),
@@ -292,7 +303,8 @@ ibus_engine_class_init (IBusEngineClass *klass)
      * Emitted when the IME is reset.
      * Implement the member function reset() in extended class to receive this signal.
      *
-     * <note><para>@user_data is not actually a valid parameter. It is displayed because of GtkDoc bug.</para></note>
+     * See also:  ibus_input_context_reset().
+     * <note><para>Argument @user_data is ignored in this function.</para></note>
      */
     engine_signals[RESET] =
         g_signal_new (I_("reset"),
@@ -311,7 +323,8 @@ ibus_engine_class_init (IBusEngineClass *klass)
      * Emitted when the IME is enabled.
      * Implement the member function set_enable() in extended class to receive this signal.
      *
-     * <note><para>@user_data is not actually a valid parameter. It is displayed because of GtkDoc bug.</para></note>
+     * See also:  ibus_input_context_enable().
+     * <note><para>Argument @user_data is ignored in this function.</para></note>
      */
     engine_signals[ENABLE] =
         g_signal_new (I_("enable"),
@@ -330,7 +343,8 @@ ibus_engine_class_init (IBusEngineClass *klass)
      * Emitted when the IME is disabled.
      * Implement the member function set_disable() in extended class to receive this signal.
      *
-     * <note><para>@user_data is not actually a valid parameter. It is displayed because of GtkDoc bug.</para></note>
+     * See also:  ibus_input_context_disable().
+     * <note><para>Argument @user_data is ignored in this function.</para></note>
      */
     engine_signals[DISABLE] =
         g_signal_new (I_("disable"),
@@ -345,11 +359,16 @@ ibus_engine_class_init (IBusEngineClass *klass)
     /**
      * IBusEngine::set-cursor-location:
      * @engine: An IBusEngine.
+     * @x: X coordinate of the cursor.
+     * @y: Y coordinate of the cursor.
+     * @w: Width of the cursor.
+     * @h: Height of the cursor.
      *
      * Emitted when the location of IME is set.
      * Implement the member function set_cursor_location() in extended class to receive this signal.
      *
-     * <note><para>@user_data is not actually a valid parameter. It is displayed because of GtkDoc bug.</para></note>
+     * See also:  ibus_input_context_set_cursor_location().
+     * <note><para>Argument @user_data is ignored in this function.</para></note>
      */
     engine_signals[SET_CURSOR_LOCATION] =
         g_signal_new (I_("set-cursor-location"),
@@ -373,7 +392,8 @@ ibus_engine_class_init (IBusEngineClass *klass)
      * Emitted when the client application capabilities is set.
      * Implement the member function set_capabilities() in extended class to receive this signal.
      *
-     * <note><para>@user_data is not actually a valid parameter. It is displayed because of GtkDoc bug.</para></note>
+     * See also:  ibus_input_context_set_capabilities().
+     * <note><para>Argument @user_data is ignored in this function.</para></note>
      */
     engine_signals[SET_CAPABILITIES] =
         g_signal_new (I_("set-capabilities"),
@@ -393,7 +413,7 @@ ibus_engine_class_init (IBusEngineClass *klass)
      * Emitted when the page-up button is pressed.
      * Implement the member function page_up() in extended class to receive this signal.
      *
-     * <note><para>@user_data is not actually a valid parameter. It is displayed because of GtkDoc bug.</para></note>
+     * <note><para>Argument @user_data is ignored in this function.</para></note>
      */
     engine_signals[PAGE_UP] =
         g_signal_new (I_("page-up"),
@@ -412,7 +432,7 @@ ibus_engine_class_init (IBusEngineClass *klass)
      * Emitted when the page-down button is pressed.
      * Implement the member function page_down() in extended class to receive this signal.
      *
-     * <note><para>@user_data is not actually a valid parameter. It is displayed because of GtkDoc bug.</para></note>
+     * <note><para>Argument @user_data is ignored in this function.</para></note>
      */
     engine_signals[PAGE_DOWN] =
         g_signal_new (I_("page-down"),
@@ -431,7 +451,7 @@ ibus_engine_class_init (IBusEngineClass *klass)
      * Emitted when the up cursor button is pressed.
      * Implement the member function cursor_up() in extended class to receive this signal.
      *
-     * <note><para>@user_data is not actually a valid parameter. It is displayed because of GtkDoc bug.</para></note>
+     * <note><para>Argument @user_data is ignored in this function.</para></note>
      */
     engine_signals[CURSOR_UP] =
         g_signal_new (I_("cursor-up"),
@@ -450,7 +470,7 @@ ibus_engine_class_init (IBusEngineClass *klass)
      * Emitted when the down cursor button is pressed.
      * Implement the member function cursor_down() in extended class to receive this signal.
      *
-     * <note><para>@user_data is not actually a valid parameter. It is displayed because of GtkDoc bug.</para></note>
+     * <note><para>Argument @user_data is ignored in this function.</para></note>
      */
     engine_signals[CURSOR_DOWN] =
         g_signal_new (I_("cursor-down"),
@@ -469,7 +489,7 @@ ibus_engine_class_init (IBusEngineClass *klass)
      * Emitted when candidate on lookup table is clicked.
      * Implement the member function candidate_clicked() in extended class to receive this signal.
      *
-     * <note><para>@user_data is not actually a valid parameter. It is displayed because of GtkDoc bug.</para></note>
+     * <note><para>Argument @user_data is ignored in this function.</para></note>
      */
     engine_signals[CANDIDATE_CLICKED] =
         g_signal_new (I_("candidate-clicked"),
@@ -491,7 +511,7 @@ ibus_engine_class_init (IBusEngineClass *klass)
      * Emitted when a property is activated or change changed.
      * Implement the member function property_activate() in extended class to receive this signal.
      *
-     * <note><para>@user_data is not actually a valid parameter. It is displayed because of GtkDoc bug.</para></note>
+     * <note><para>Argument @user_data is ignored in this function.</para></note>
      */
     engine_signals[PROPERTY_ACTIVATE] =
         g_signal_new (I_("property-activate"),
@@ -512,7 +532,7 @@ ibus_engine_class_init (IBusEngineClass *klass)
      * Emitted when a property is shown.
      * Implement the member function property_side() in extended class to receive this signal.
      *
-     * <note><para>@user_data is not actually a valid parameter. It is displayed because of GtkDoc bug.</para></note>
+     * <note><para>Argument @user_data is ignored in this function.</para></note>
      */
     engine_signals[PROPERTY_SHOW] =
         g_signal_new (I_("property-show"),
@@ -532,7 +552,7 @@ ibus_engine_class_init (IBusEngineClass *klass)
      * Emitted when a property is hidden.
      * Implement the member function property_hide() in extended class to receive this signal.
      *
-     * <note><para>@user_data is not actually a valid parameter. It is displayed because of GtkDoc bug.</para></note>
+     * <note><para>Argument @user_data is ignored in this function.</para></note>
      */
     engine_signals[PROPERTY_HIDE] =
         g_signal_new (I_("property-hide"),
@@ -678,24 +698,26 @@ ibus_engine_ibus_message (IBusEngine     *engine,
     }
 
     if (ibus_message_is_method_call (message, IBUS_INTERFACE_ENGINE, "ProcessKeyEvent")) {
-        guint keyval, state;
+        guint keyval, keycode, state;
         gboolean retval;
         IBusError *error = NULL;
 
         retval = ibus_message_get_args (message,
                                         &error,
                                         G_TYPE_UINT, &keyval,
+                                        G_TYPE_UINT, &keycode,
                                         G_TYPE_UINT, &state,
                                         G_TYPE_INVALID);
 
         if (!retval)
-            goto _keypress_fail;
+            goto _keypress2_fail;
 
         retval = FALSE;
         g_signal_emit (engine,
                        engine_signals[PROCESS_KEY_EVENT],
                        0,
                        keyval,
+                       keycode,
                        state,
                        &retval);
 
@@ -707,15 +729,16 @@ ibus_engine_ibus_message (IBusEngine     *engine,
         ibus_message_unref (return_message);
         return TRUE;
 
-    _keypress_fail:
+    _keypress2_fail:
         error_message = ibus_message_new_error_printf (message,
                         DBUS_ERROR_INVALID_ARGS,
-                        "%s.%s: Can not match signature (uu) of method",
-                        IBUS_INTERFACE_ENGINE, "ProcessKeyEvent");
+                        "%s.%s: Can not match signature (ubu) of method",
+                        IBUS_INTERFACE_ENGINE, "ProcessKeyEvent2");
         ibus_connection_send (connection, error_message);
         ibus_message_unref (error_message);
         return TRUE;
     }
+
     else if (ibus_message_is_method_call (message, IBUS_INTERFACE_ENGINE, "CandidateClicked")) {
         guint index, button, state;
         gboolean retval;
@@ -938,9 +961,9 @@ ibus_engine_ibus_message (IBusEngine     *engine,
 static gboolean
 ibus_engine_process_key_event (IBusEngine *engine,
                                guint       keyval,
+                               guint       keycode,
                                guint       state)
 {
-    // g_debug ("process-key-event (%d, %d)", keyval, state);
     return FALSE;
 }
 
@@ -1194,11 +1217,13 @@ void ibus_engine_hide_lookup_table (IBusEngine *engine)
 
 void ibus_engine_forward_key_event (IBusEngine      *engine,
                                     guint            keyval,
+                                    guint            keycode,
                                     guint            state)
 {
     _send_signal (engine,
                   "ForwardKeyEvent",
                   G_TYPE_UINT, &keyval,
+                  G_TYPE_UINT, &keycode,
                   G_TYPE_UINT, &state,
                   G_TYPE_INVALID);
 }
