@@ -128,6 +128,12 @@ ibus_input_context_class_init (IBusInputContextClass *klass)
     proxy_class->ibus_signal = ibus_input_context_ibus_signal;
 
     /* install signals */
+    /**
+     * IBusInputContext::enabled:
+     * @context: An IBusInputContext.
+     *
+     * Emitted when an IME is enabled.
+     */
     context_signals[ENABLED] =
         g_signal_new (I_("enabled"),
             G_TYPE_FROM_CLASS (klass),
@@ -137,6 +143,12 @@ ibus_input_context_class_init (IBusInputContextClass *klass)
             ibus_marshal_VOID__VOID,
             G_TYPE_NONE, 0);
 
+    /**
+     * IBusInputContext::disabled:
+     * @context: An IBusInputContext.
+     *
+     * Emitted when an IME is disabled.
+     */
     context_signals[DISABLED] =
         g_signal_new (I_("disabled"),
             G_TYPE_FROM_CLASS (klass),
@@ -146,6 +158,13 @@ ibus_input_context_class_init (IBusInputContextClass *klass)
             ibus_marshal_VOID__VOID,
             G_TYPE_NONE, 0);
 
+    /**
+     * IBusInputContext::commit-text:
+     * @context: An IBusInputContext.
+     * @text: Text to be committed.
+     *
+     * Emitted when the text is going to be committed.
+     */
     context_signals[COMMIT_TEXT] =
         g_signal_new (I_("commit-text"),
             G_TYPE_FROM_CLASS (klass),
@@ -157,18 +176,37 @@ ibus_input_context_class_init (IBusInputContextClass *klass)
             1,
             IBUS_TYPE_TEXT);
 
+    /**
+     * IBusInputContext::forward-key-event:
+     * @context: An IBusInputContext.
+     * @keyval: Key symbol of the keyboard event.
+     * @keycode: Key symbol of the keyboard event.
+     * @modifiers: Key modifier flags.
+     *
+     * Emitted to forward key event from IME to client of IME.
+     */
     context_signals[FORWARD_KEY_EVENT] =
         g_signal_new (I_("forward-key-event"),
             G_TYPE_FROM_CLASS (klass),
             G_SIGNAL_RUN_LAST,
             0,
             NULL, NULL,
-            ibus_marshal_VOID__UINT_UINT,
+            ibus_marshal_VOID__UINT_UINT_UINT,
             G_TYPE_NONE,
-            2,
+            3,
+            G_TYPE_UINT,
             G_TYPE_UINT,
             G_TYPE_UINT);
 
+    /**
+     * IBusInputContext::update-preedit-text:
+     * @context: An IBusInputContext.
+     * @text: Text to be updated.
+     * @cursor_pos: Cursor position.
+     * @visible: Whether the update is visible.
+     *
+     * Emitted to update preedit text.
+     */
     context_signals[UPDATE_PREEDIT_TEXT] =
         g_signal_new (I_("update-preedit-text"),
             G_TYPE_FROM_CLASS (klass),
@@ -182,6 +220,12 @@ ibus_input_context_class_init (IBusInputContextClass *klass)
             G_TYPE_UINT,
             G_TYPE_BOOLEAN);
 
+    /**
+     * IBusInputContext::show-preedit-text:
+     * @context: An IBusInputContext.
+     *
+     * Emitted to show preedit text.
+     */
     context_signals[SHOW_PREEDIT_TEXT] =
         g_signal_new (I_("show-preedit-text"),
             G_TYPE_FROM_CLASS (klass),
@@ -191,6 +235,12 @@ ibus_input_context_class_init (IBusInputContextClass *klass)
             ibus_marshal_VOID__VOID,
             G_TYPE_NONE, 0);
 
+    /**
+     * IBusInputContext::hide-preedit-text:
+     * @context: An IBusInputContext.
+     *
+     * Emitted to hide preedit text.
+     */
     context_signals[HIDE_PREEDIT_TEXT] =
         g_signal_new (I_("hide-preedit-text"),
             G_TYPE_FROM_CLASS (klass),
@@ -200,6 +250,12 @@ ibus_input_context_class_init (IBusInputContextClass *klass)
             ibus_marshal_VOID__VOID,
             G_TYPE_NONE, 0);
 
+    /**
+     * IBusInputContext::update-auxiliary-text:
+     * @context: An IBusInputContext.
+     *
+     * Emitted to hide auxilary text.
+     */
     context_signals[UPDATE_AUXILIARY_TEXT] =
         g_signal_new (I_("update-auxiliary-text"),
             G_TYPE_FROM_CLASS (klass),
@@ -211,6 +267,12 @@ ibus_input_context_class_init (IBusInputContextClass *klass)
             IBUS_TYPE_TEXT,
             G_TYPE_BOOLEAN);
 
+    /**
+     * IBusInputContext::show-auxiliary-text:
+     * @context: An IBusInputContext.
+     *
+     * Emitted to show auxiliary text.
+     */
     context_signals[SHOW_AUXILIARY_TEXT] =
         g_signal_new (I_("show-auxiliary-text"),
             G_TYPE_FROM_CLASS (klass),
@@ -220,6 +282,12 @@ ibus_input_context_class_init (IBusInputContextClass *klass)
             ibus_marshal_VOID__VOID,
             G_TYPE_NONE, 0);
 
+    /**
+     * IBusInputContext::hide-auxiliary-text:
+     * @context: An IBusInputContext.
+     *
+     * Emitted to hide auxiliary text.
+     */
     context_signals[HIDE_AUXILIARY_TEXT] =
         g_signal_new (I_("hide-auxiliary-text"),
             G_TYPE_FROM_CLASS (klass),
@@ -229,6 +297,14 @@ ibus_input_context_class_init (IBusInputContextClass *klass)
             ibus_marshal_VOID__VOID,
             G_TYPE_NONE, 0);
 
+    /**
+     * IBusInputContext::update-lookup-table:
+     * @context: An IBusInputContext.
+     * @table: An IBusLookupTable to be updated.
+     * @visible: Whether the table should be visible.
+     *
+     * Emitted to update lookup table.
+     */
     context_signals[UPDATE_LOOKUP_TABLE] =
         g_signal_new (I_("update-lookup-table"),
             G_TYPE_FROM_CLASS (klass),
@@ -240,6 +316,12 @@ ibus_input_context_class_init (IBusInputContextClass *klass)
             IBUS_TYPE_LOOKUP_TABLE,
             G_TYPE_BOOLEAN);
 
+    /**
+     * IBusInputContext::show-lookup-table:
+     * @context: An IBusInputContext.
+     *
+     * Emitted to show lookup table.
+     */
     context_signals[SHOW_LOOKUP_TABLE] =
         g_signal_new (I_("show-lookup-table"),
             G_TYPE_FROM_CLASS (klass),
@@ -249,6 +331,12 @@ ibus_input_context_class_init (IBusInputContextClass *klass)
             ibus_marshal_VOID__VOID,
             G_TYPE_NONE, 0);
 
+    /**
+     * IBusInputContext::hide-lookup-table:
+     * @context: An IBusInputContext.
+     *
+     * Emitted to hide lookup table.
+     */
     context_signals[HIDE_LOOKUP_TABLE] =
         g_signal_new (I_("hide-lookup-table"),
             G_TYPE_FROM_CLASS (klass),
@@ -258,6 +346,12 @@ ibus_input_context_class_init (IBusInputContextClass *klass)
             ibus_marshal_VOID__VOID,
             G_TYPE_NONE, 0);
 
+    /**
+     * IBusInputContext::page-up-lookup-table:
+     * @context: An IBusInputContext.
+     *
+     * Emitted to view the previous page of lookup table.
+     */
     context_signals[PAGE_UP_LOOKUP_TABLE] =
         g_signal_new (I_("page-up-lookup-table"),
             G_TYPE_FROM_CLASS (klass),
@@ -267,6 +361,12 @@ ibus_input_context_class_init (IBusInputContextClass *klass)
             ibus_marshal_VOID__VOID,
             G_TYPE_NONE, 0);
 
+    /**
+     * IBusInputContext::page-down-lookup-table:
+     * @context: An IBusInputContext.
+     *
+     * Emitted to view the next page of lookup table.
+     */
     context_signals[PAGE_DOWN_LOOKUP_TABLE] =
         g_signal_new (I_("page-down-lookup-table"),
             G_TYPE_FROM_CLASS (klass),
@@ -276,6 +376,12 @@ ibus_input_context_class_init (IBusInputContextClass *klass)
             ibus_marshal_VOID__VOID,
             G_TYPE_NONE, 0);
 
+    /**
+     * IBusInputContext::cursor-up-lookup-table:
+     * @context: An IBusInputContext.
+     *
+     * Emitted to select previous candidate of lookup table.
+     */
     context_signals[CURSOR_UP_LOOKUP_TABLE] =
         g_signal_new (I_("cursor-up-lookup-table"),
             G_TYPE_FROM_CLASS (klass),
@@ -285,6 +391,12 @@ ibus_input_context_class_init (IBusInputContextClass *klass)
             ibus_marshal_VOID__VOID,
             G_TYPE_NONE, 0);
 
+    /**
+     * IBusInputContext::cursor-down-lookup-table:
+     * @context: An IBusInputContext.
+     *
+     * Emitted to select next candidate of lookup table.
+     */
     context_signals[CURSOR_DOWN_LOOKUP_TABLE] =
         g_signal_new (I_("cursor-down-lookup-table"),
             G_TYPE_FROM_CLASS (klass),
@@ -295,6 +407,13 @@ ibus_input_context_class_init (IBusInputContextClass *klass)
             G_TYPE_NONE,
             0);
 
+    /**
+     * IBusInputContext::register-properties:
+     * @context: An IBusInputContext.
+     * @props: An IBusPropList that contains properties.
+     *
+     * Emitted to register the properties in @props.
+     */
     context_signals[REGISTER_PROPERTIES] =
         g_signal_new (I_("register-properties"),
             G_TYPE_FROM_CLASS (klass),
@@ -306,6 +425,13 @@ ibus_input_context_class_init (IBusInputContextClass *klass)
             1,
             IBUS_TYPE_PROP_LIST);
 
+    /**
+     * IBusInputContext::update-property:
+     * @context: An IBusInputContext.
+     * @prop: The IBusProperty to be updated.
+     *
+     * Emitted to update the property @prop.
+     */
     context_signals[UPDATE_PROPERTY] =
         g_signal_new (I_("update-property"),
             G_TYPE_FROM_CLASS (klass),
@@ -426,12 +552,14 @@ ibus_input_context_ibus_signal (IBusProxy           *proxy,
                                      IBUS_INTERFACE_INPUT_CONTEXT,
                                      "ForwardKeyEvent")) {
         guint32 keyval;
+        guint32 keycode;
         guint32 state;
         gboolean retval;
 
         retval = ibus_message_get_args (message,
                                         &error,
                                         G_TYPE_UINT, &keyval,
+                                        G_TYPE_UINT, &keycode,
                                         G_TYPE_UINT, &state,
                                         G_TYPE_INVALID);
 
@@ -441,6 +569,7 @@ ibus_input_context_ibus_signal (IBusProxy           *proxy,
                        context_signals[FORWARD_KEY_EVENT],
                        0,
                        keyval,
+                       keycode,
                        state | IBUS_FORWARD_MASK);
     }
     else if (ibus_message_is_signal (message,
@@ -541,9 +670,11 @@ failed:
     return FALSE;
 }
 
+
 gboolean
 ibus_input_context_process_key_event (IBusInputContext *context,
                                       guint32           keyval,
+                                      guint32           keycode,
                                       guint32           state)
 {
     g_assert (IBUS_IS_INPUT_CONTEXT (context));
@@ -565,6 +696,7 @@ ibus_input_context_process_key_event (IBusInputContext *context,
                                          -1,
                                          &error,
                                          G_TYPE_UINT, &keyval,
+                                         G_TYPE_UINT, &keycode,
                                          G_TYPE_UINT, &state,
                                          G_TYPE_INVALID);
     if (!retval) {
@@ -593,7 +725,6 @@ ibus_input_context_process_key_event (IBusInputContext *context,
         retval = FALSE;
     }
     else {
-
         if (!ibus_message_get_args (reply_message,
                                     &error,
                                     G_TYPE_BOOLEAN, &retval,
