@@ -438,6 +438,7 @@ ibus_message_get_args_valist (IBusMessage *message,
         retval = ibus_message_iter_get (&iter, type, value);
         if (!retval)
             goto _failed;
+        ibus_message_iter_next (&iter);
         i ++;
         type = va_arg (va_args, GType);
     }
@@ -503,18 +504,18 @@ ibus_message_iter_append (IBusMessageIter *iter,
             v = * (guint *)value;
             return dbus_message_iter_append_basic (iter, DBUS_TYPE_UINT32, &v);
         }
-    case G_TYPE_ULONG:
-        {
-            dbus_uint64_t v;
-            v = * (gulong *)value;
-            return dbus_message_iter_append_basic (iter, DBUS_TYPE_UINT64, &v);
-        }
     case G_TYPE_LONG:
         {
             dbus_int64_t v;
             v = * (glong *)value;
             return dbus_message_iter_append_basic (iter, DBUS_TYPE_INT64, &v);
         }
+    case G_TYPE_ULONG:
+        {
+            dbus_uint64_t v;
+            v = * (gulong *)value;
+            return dbus_message_iter_append_basic (iter, DBUS_TYPE_UINT64, &v);
+        }
     case G_TYPE_BOOLEAN:
         {
             dbus_bool_t v;
@@ -571,6 +572,152 @@ ibus_message_iter_append (IBusMessageIter *iter,
     return FALSE;
 }
 
+static gboolean
+gtype_is_basic (GType type)
+{
+    switch (type) {
+    case G_TYPE_CHAR:
+    case G_TYPE_INT:
+    case G_TYPE_UINT:
+    case G_TYPE_INT64:
+    case G_TYPE_UINT64:
+    case G_TYPE_BOOLEAN:
+    case G_TYPE_DOUBLE:
+    case G_TYPE_STRING:
+        return TRUE;
+    default:
+        if (type == IBUS_TYPE_OBJECT_PATH)
+            return TRUE;
+        return FALSE;
+    }
+}
+
+gboolean
+ibus_message_iter_copy_data (IBusMessageIter *dst,
+                             IBusMessageIter *src)
+{
+    GType type;
+    gboolean retval;
+
+    type = ibus_message_iter_get_arg_type (src);
+
+    g_return_val_if_fail (type != G_TYPE_INVALID, FALSE);
+
+    if (gtype_is_basic (type)) {
+        gchar data[16];
+        ibus_message_iter_get_basic (src, data);
+        retval = ibus_message_iter_append (dst, type, data);
+        g_return_val_if_fail (retval, FALSE);
+        return TRUE;
+    }
+
+    if (type == IBUS_TYPE_VARIANT) {
+        IBusMessageIter subdst, subsrc;
+        gchar *signature;
+
+        retval = ibus_message_iter_recurse (src, IBUS_TYPE_VARIANT, &subsrc);
+        g_return_val_if_fail (retval, FALSE);
+
+        signature = dbus_message_iter_get_signature (&subsrc);
+        g_return_val_if_fail (signature != NULL, FALSE);
+        retval = ibus_message_iter_open_container (dst,
+                                                   IBUS_TYPE_VARIANT,
+                                                   signature,
+                                                   &subdst);
+        dbus_free (signature);
+        g_return_val_if_fail (retval, FALSE);
+
+        retval = ibus_message_iter_copy_data (&subdst, &subsrc);
+        g_return_val_if_fail (retval, FALSE);
+
+        retval = ibus_message_iter_close_container (dst, &subdst);
+        g_return_val_if_fail (retval, FALSE);
+
+        return TRUE;
+    }
+    else if (type == IBUS_TYPE_ARRAY) {
+        IBusMessageIter subdst, subsrc;
+        gchar *signature;
+
+        retval = ibus_message_iter_recurse (src, IBUS_TYPE_ARRAY, &subsrc);
+        g_return_val_if_fail (retval, FALSE);
+
+        signature = dbus_message_iter_get_signature (src);
+        g_return_val_if_fail (signature != NULL, FALSE);
+
+        retval = ibus_message_iter_open_container (dst,
+                                                   IBUS_TYPE_ARRAY,
+                                                   signature + 1,
+                                                   &subdst);
+        dbus_free (signature);
+        g_return_val_if_fail (retval, FALSE);
+
+        while (ibus_message_iter_get_arg_type (&subsrc) != G_TYPE_INVALID) {
+            retval = ibus_message_iter_copy_data (&subdst, &subsrc);
+            g_return_val_if_fail (retval, FALSE);
+            ibus_message_iter_next (&subsrc);
+        }
+
+        retval = ibus_message_iter_close_container (dst, &subdst);
+        g_return_val_if_fail (retval, FALSE);
+
+        return TRUE;
+    }
+    else if (type == IBUS_TYPE_STRUCT) {
+        IBusMessageIter subdst, subsrc;
+
+        retval = ibus_message_iter_recurse (src, IBUS_TYPE_STRUCT, &subsrc);
+        g_return_val_if_fail (retval, FALSE);
+
+        retval = ibus_message_iter_open_container (dst,
+                                                   IBUS_TYPE_STRUCT,
+                                                   NULL,
+                                                   &subdst);
+        g_return_val_if_fail (retval, FALSE);
+
+        while (ibus_message_iter_get_arg_type (&subsrc) != G_TYPE_INVALID) {
+            retval = ibus_message_iter_copy_data (&subdst, &subsrc);
+            ibus_message_iter_next (&subsrc);
+            g_return_val_if_fail (retval, FALSE);
+        }
+
+        retval = ibus_message_iter_close_container (dst, &subdst);
+        g_return_val_if_fail (retval, FALSE);
+
+        return TRUE;
+    }
+    else if (type == IBUS_TYPE_DICT_ENTRY) {
+        IBusMessageIter subdst, subsrc;
+
+        retval = ibus_message_iter_recurse (src, IBUS_TYPE_DICT_ENTRY, &subsrc);
+        g_return_val_if_fail (retval, FALSE);
+
+        retval = ibus_message_iter_open_container (dst,
+                                                   IBUS_TYPE_DICT_ENTRY,
+                                                   NULL,
+                                                   &subdst);
+        g_return_val_if_fail (retval, FALSE);
+
+        /* copy key */
+        retval = ibus_message_iter_copy_data (&subdst, &subsrc);
+        g_return_val_if_fail (retval, FALSE);
+        ibus_message_iter_next (&subsrc);
+
+        /* copy value */
+        retval = ibus_message_iter_copy_data (&subdst, &subsrc);
+        g_return_val_if_fail (retval, FALSE);
+        ibus_message_iter_next (&subsrc);
+
+        retval = ibus_message_iter_close_container (dst, &subdst);
+        g_return_val_if_fail (retval, FALSE);
+
+        return TRUE;
+    }
+
+    return FALSE;
+
+}
+
 gboolean
 ibus_message_iter_init (IBusMessage     *message,
                         IBusMessageIter *iter)
@@ -583,26 +730,12 @@ ibus_message_iter_get_basic (IBusMessageIter *iter,
                              gpointer         value)
 {
     dbus_message_iter_get_basic (iter, value);
-    dbus_message_iter_next (iter);
 }
 
 gboolean
 ibus_message_iter_get (IBusMessageIter *iter,
                        GType            type,
                        gpointer         value)
-{
-    gboolean retval;
-
-    retval = ibus_message_iter_peek (iter, type, value);
-    ibus_message_iter_next (iter);
-
-    return retval;
-}
-
-gboolean
-ibus_message_iter_peek (IBusMessageIter *iter,
-                        GType            type,
-                        gpointer         value)
 {
     g_assert (iter != NULL);
     g_assert (type != G_TYPE_INVALID);
@@ -636,6 +769,25 @@ ibus_message_iter_peek (IBusMessageIter *iter,
             *(guint *) value = (guint) v;
             return TRUE;
         }
+    case G_TYPE_LONG:
+        {
+            dbus_int64_t v;
+            if (dbus_message_iter_get_arg_type (iter) != DBUS_TYPE_INT32)
+                return FALSE;
+            dbus_message_iter_get_basic (iter, &v);
+            *(glong *) value = (glong) v;
+            return TRUE;
+        }
+    case G_TYPE_ULONG:
+        {
+            dbus_uint64_t v;
+            if (dbus_message_iter_get_arg_type (iter) != DBUS_TYPE_UINT32)
+                return FALSE;
+            dbus_message_iter_get_basic (iter, &v);
+            *(gulong *) value = (gulong) v;
+            return TRUE;
+        }
+
     case G_TYPE_BOOLEAN:
         {
             dbus_bool_t v;
