@@ -97,15 +97,17 @@ ibus_property_init (IBusProperty *prop)
     prop->sensitive = FALSE;
     prop->visible = FALSE;
     prop->state = 0;
+
+    prop->sub_props = NULL;
 }
 
 static void
 ibus_property_destroy (IBusProperty *prop)
 {
     g_free (prop->key);
-    g_free (prop->icon);
-
     prop->key = NULL;
+
+    g_free (prop->icon);
     prop->icon = NULL;
 
     if (prop->label) {
@@ -118,6 +120,11 @@ ibus_property_destroy (IBusProperty *prop)
         prop->tooltip = NULL;
     }
 
+    if (prop->sub_props) {
+        g_object_unref (prop->sub_props);
+        prop->sub_props = NULL;
+    }
+
     IBUS_OBJECT_CLASS (parent_class)->destroy ((IBusObject *)prop);
 }
 
@@ -174,32 +181,41 @@ ibus_property_deserialize (IBusProperty    *prop,
 
     retval = ibus_message_iter_get (iter, G_TYPE_STRING, &p);
     g_return_val_if_fail (retval, FALSE);
+    ibus_message_iter_next (iter);
     prop->key = g_strdup (p);
 
     retval = ibus_message_iter_get (iter, G_TYPE_UINT, &prop->type);
     g_return_val_if_fail (retval, FALSE);
+    ibus_message_iter_next (iter);
 
     retval = ibus_message_iter_get (iter, IBUS_TYPE_TEXT, &prop->label);
     g_return_val_if_fail (retval, FALSE);
+    ibus_message_iter_next (iter);
 
     retval = ibus_message_iter_get (iter, G_TYPE_STRING, &p);
     g_return_val_if_fail (retval, FALSE);
+    ibus_message_iter_next (iter);
     prop->icon = g_strdup (p);
 
     retval = ibus_message_iter_get (iter, IBUS_TYPE_TEXT, &prop->tooltip);
     g_return_val_if_fail (retval, FALSE);
+    ibus_message_iter_next (iter);
 
     retval = ibus_message_iter_get (iter, G_TYPE_BOOLEAN, &prop->sensitive);
     g_return_val_if_fail (retval, FALSE);
+    ibus_message_iter_next (iter);
 
     retval = ibus_message_iter_get (iter, G_TYPE_BOOLEAN, &prop->visible);
     g_return_val_if_fail (retval, FALSE);
+    ibus_message_iter_next (iter);
 
     retval = ibus_message_iter_get (iter, G_TYPE_UINT, &prop->state);
     g_return_val_if_fail (retval, FALSE);
+    ibus_message_iter_next (iter);
 
     retval = ibus_message_iter_get (iter, IBUS_TYPE_PROP_LIST, &prop->sub_props);
     g_return_val_if_fail (retval, FALSE);
+    ibus_message_iter_next (iter);
 
     return TRUE;
 }
@@ -254,39 +270,21 @@ ibus_property_new (const gchar   *key,
     g_return_val_if_fail (type >= PROP_TYPE_NORMAL &&
                           type <= PROP_TYPE_SEPARATOR,
                           NULL);
-    g_return_val_if_fail (label == NULL || IBUS_IS_TEXT (label), NULL);
-    g_return_val_if_fail (tooltip == NULL || IBUS_IS_TEXT (tooltip), NULL);
-    g_return_val_if_fail (state == PROP_STATE_UNCHECKED ||
-                          state == PROP_STATE_CHECKED ||
-                          state == PROP_STATE_INCONSISTENT,
-                          NULL);
 
     IBusProperty *prop;
 
     prop = (IBusProperty *)g_object_new (IBUS_TYPE_PROPERTY, NULL);
 
     prop->key = g_strdup (key);
-    prop->icon = g_strdup (icon != NULL ? icon : "");
     prop->type = type;
-
-    if (label)
-        prop->label = (IBusText *) g_object_ref (label);
-    else
-        prop->label = ibus_text_new_from_static_string ("");
-
-    if (tooltip)
-        prop->tooltip = (IBusText *) g_object_ref (tooltip);
-    else
-        prop->tooltip = ibus_text_new_from_static_string ("");
-
-    prop->sensitive = sensitive;
-    prop->visible = visible;
-    prop->state = state;
-
-    if (prop_list)
-        prop->sub_props = g_object_ref (prop_list);
-    else
-        prop->sub_props = ibus_prop_list_new ();
+    
+    ibus_property_set_label (prop, label);
+    ibus_property_set_icon (prop, icon);
+    ibus_property_set_tooltip (prop, tooltip);
+    ibus_property_set_sensitive (prop, sensitive);
+    ibus_property_set_visible (prop, visible);
+    ibus_property_set_state (prop, state);
+    ibus_property_set_sub_props (prop, prop_list);
 
     return prop;
 }
@@ -296,16 +294,56 @@ ibus_property_set_label (IBusProperty *prop,
                          IBusText     *label)
 {
     g_assert (IBUS_IS_PROPERTY (prop));
+    g_return_if_fail (label == NULL || IBUS_IS_TEXT (label));
 
     if (prop->label) {
         g_object_unref (prop->label);
     }
 
     if (label == NULL) {
-        label = ibus_text_new_from_static_string ("");
+        prop->label = ibus_text_new_from_static_string ("");
+    }
+    else {
+        prop->label = g_object_ref (label);
+    }
+}
+
+void
+ibus_property_set_icon (IBusProperty *prop,
+                        const gchar  *icon)
+{
+    g_assert (IBUS_IS_PROPERTY (prop));
+
+    g_free (prop->icon);
+    prop->icon = NULL;
+    prop->icon = g_strdup (icon != NULL ? icon : "");
+}
+
+void
+ibus_property_set_tooltip (IBusProperty *prop,
+                           IBusText     *tooltip)
+{
+    g_assert (IBUS_IS_PROPERTY (prop));
+    g_return_if_fail (tooltip == NULL || IBUS_IS_TEXT (tooltip));
+
+    if (prop->tooltip) {
+        g_object_unref (prop->tooltip);
     }
 
-    prop->label = g_object_ref (label);
+    if (tooltip == NULL) {
+        prop->tooltip = ibus_text_new_from_static_string ("");
+    }
+    else {
+        prop->tooltip = g_object_ref (tooltip);
+    }
+}
+
+void
+ibus_property_set_sensitive (IBusProperty *prop,
+                             gboolean      sensitive)
+{
+    g_assert (IBUS_IS_PROPERTY (prop));
+    prop->sensitive = sensitive;
 }
 
 void
@@ -316,6 +354,18 @@ ibus_property_set_visible (IBusProperty *prop,
     prop->visible = visible;
 }
 
+void
+ibus_property_set_state (IBusProperty  *prop,
+                         IBusPropState  state)
+{
+    g_assert (IBUS_IS_PROPERTY (prop));
+    g_return_if_fail (state == PROP_STATE_UNCHECKED ||
+                      state == PROP_STATE_CHECKED ||
+                      state == PROP_STATE_INCONSISTENT);
+
+    prop->state = state;
+}
+
 void
 ibus_property_set_sub_props (IBusProperty *prop,
                              IBusPropList *prop_list)
@@ -418,14 +468,15 @@ ibus_prop_list_init (IBusPropList *prop_list)
 static void
 ibus_prop_list_destroy (IBusPropList *prop_list)
 {
-    IBusProperty **ps, **p;
-    p = ps = (IBusProperty **) g_array_free (prop_list->properties, FALSE);
+    IBusProperty **p;
+    gint i;
 
-    while (*p != NULL) {
-        g_object_unref (*p);
-        p ++;
+    p = (IBusProperty **) g_array_free (prop_list->properties, FALSE);
+
+    for (i = 0; p[i] != NULL; i++) {
+        g_object_unref (p[i]);
     }
-    g_free (ps);
+    g_free (p);
 
     IBUS_OBJECT_CLASS (parent_class)->destroy ((IBusObject *) prop_list);
 }
@@ -478,6 +529,7 @@ ibus_prop_list_deserialize (IBusPropList    *prop_list,
     while (ibus_message_iter_get_arg_type (&array_iter) != G_TYPE_INVALID) {
         retval = ibus_message_iter_get (&array_iter, IBUS_TYPE_PROPERTY, &object);
         g_return_val_if_fail (retval, FALSE);
+        ibus_message_iter_next (&array_iter);
 
         ibus_prop_list_append (prop_list, (IBusProperty *)object);
         g_object_unref (object);
