@@ -24,6 +24,7 @@ import gtk.gdk as gdk
 import gobject
 
 class Handle(gtk.EventBox):
+    __gtype_name__ = "IBusHandle"
     __gsignals__ = {
         "move-begin" : (
             gobject.SIGNAL_RUN_LAST,
@@ -117,6 +118,3 @@ class Handle(gtk.EventBox):
                     gtk.ORIENTATION_VERTICAL)
         return True
 
-gobject.type_register(Handle, "IBusHandle")
-
-
