@@ -37,6 +37,9 @@ from gettext import dgettext
 _  = lambda a : dgettext("ibus", a)
 N_ = lambda a : a
 
+ICON_OFF = "ibus-off"
+ICON_ENGINE = "ibus-engine"
+
 def show_uri(screen, link):
     try:
         gtk.show_uri(screen, link, 0)
@@ -53,7 +56,8 @@ gtk.about_dialog_set_url_hook(url_hook, None)
 gtk.about_dialog_set_email_hook(email_hook, None)
 
 class Panel(ibus.PanelBase):
-    def __init__ (self, bus):
+    __gtype_name__ = "IBusPanel"
+    def __init__(self, bus):
         super(Panel, self).__init__(bus)
         self.__bus = bus
         self.__config = self.__bus.get_config()
@@ -61,8 +65,7 @@ class Panel(ibus.PanelBase):
         self.__setup_pid = 0
         self.__prefix = os.getenv("IBUS_PREFIX")
         self.__data_dir = path.join(self.__prefix, "share", "ibus")
-        self.__icons_dir = path.join(self.__data_dir, "icons")
-        self.__ibus_icon = path.join(self.__icons_dir, "ibus.svg")
+        # self.__icons_dir = path.join(self.__data_dir, "icons")
         self.__setup_cmd = path.join(self.__prefix, "bin", "ibus-setup")
 
         # hanlder signal
@@ -74,8 +77,8 @@ class Panel(ibus.PanelBase):
         # self.__bus.config_add_watch("panel")
 
         # add icon search path
-        icon_theme = gtk.icon_theme_get_default()
-        icon_theme.prepend_search_path(self.__icons_dir)
+        # icon_theme = gtk.icon_theme_get_default()
+        # icon_theme.prepend_search_path(self.__icons_dir)
 
         self.__language_bar = LanguageBar()
         self.__language_bar.connect("property-activate",
@@ -103,7 +106,7 @@ class Panel(ibus.PanelBase):
         self.__status_icon = gtk.StatusIcon()
         self.__status_icon.connect("popup-menu", self.__status_icon_popup_menu_cb)
         self.__status_icon.connect("activate", self.__status_icon_activate_cb)
-        self.__status_icon.set_from_file(self.__ibus_icon)
+        self.__status_icon.set_from_icon_name(ICON_OFF)
         self.__status_icon.set_tooltip(_("IBus input method framework"))
         self.__status_icon.set_visible(True)
 
@@ -178,7 +181,7 @@ class Panel(ibus.PanelBase):
 
     def __set_im_icon(self, icon_name):
         if not icon_name:
-            icon_name = "engine-default"
+            icon_name = ICON_ENGINE
         self.__language_bar.set_im_icon(icon_name)
         if icon_name.startswith("/"):
             self.__status_icon.set_from_file(icon_name)
@@ -195,7 +198,7 @@ class Panel(ibus.PanelBase):
         self.__language_bar.set_enabled(enabled)
 
         if not enabled:
-            self.__set_im_icon(self.__ibus_icon)
+            self.__set_im_icon(ICON_OFF)
             self.__set_im_name(None)
         else:
             engine = self.__focus_ic.get_engine()
@@ -203,7 +206,7 @@ class Panel(ibus.PanelBase):
                 self.__set_im_icon(engine.icon)
                 self.__set_im_name(engine.longname)
             else:
-                self.__set_im_icon(self.__ibus_icon)
+                self.__set_im_icon(ICON_OFF)
                 self.__set_im_name(None)
         self.__language_bar.focus_in()
 
@@ -212,7 +215,7 @@ class Panel(ibus.PanelBase):
         self.__focus_ic = None
         self.__language_bar.set_enabled(False)
         self.__language_bar.focus_out()
-        self.__set_im_icon(self.__ibus_icon)
+        self.__set_im_icon(ICON_OFF)
         self.__set_im_name(None)
 
     def state_changed(self):
@@ -224,7 +227,7 @@ class Panel(ibus.PanelBase):
 
         if enabled == False:
             self.reset()
-            self.__set_im_icon(self.__ibus_icon)
+            self.__set_im_icon(ICON_OFF)
             self.__set_im_name(None)
         else:
             engine = self.__focus_ic.get_engine()
@@ -232,7 +235,7 @@ class Panel(ibus.PanelBase):
                 self.__set_im_icon(engine.icon)
                 self.__set_im_name(engine.longname)
             else:
-                self.__set_im_icon(self.__ibus_icon)
+                self.__set_im_icon(ICON_OFF)
                 self.__set_im_name(None)
 
 
@@ -265,7 +268,7 @@ class Panel(ibus.PanelBase):
         font_name = unicode(font_name, "utf-8")
         custom_font =  self.__config.get_value("panel", "custom_font", font_name)
         style_string = 'style "custom-font" { font_name="%s" }\n' \
-            'class "IBusPanelLabel" style "custom-font"\n'
+            'class "IBusCandidateLabel" style "custom-font"\n'
         if use_custom_font:
             style_string = style_string % custom_font
             gtk.rc_parse_string(style_string)
@@ -344,9 +347,9 @@ class Panel(ibus.PanelBase):
                 engine = tmp[lang][0]
                 item = gtk.ImageMenuItem("%s - %s" % (lang, engine.longname))
                 if engine.icon:
-                    item.set_image (_icon.IconWidget(engine.icon, size[0]))
+                    item.set_image(_icon.IconWidget(engine.icon, size[0]))
                 else:
-                    item.set_image (_icon.IconWidget("engine-default", size[0]))
+                    item.set_image(_icon.IconWidget(ICON_ENGINE, size[0]))
                 item.connect("activate", self.__im_menu_item_activate_cb, engine)
                 menu.add(item)
             else:
@@ -357,16 +360,16 @@ class Panel(ibus.PanelBase):
                 for engine in tmp[lang]:
                     item = gtk.ImageMenuItem(engine.longname)
                     if engine.icon:
-                        item.set_image (_icon.IconWidget(engine.icon, size[0]))
+                        item.set_image(_icon.IconWidget(engine.icon, size[0]))
                     else:
-                        item.set_image (_icon.IconWidget("engine-default", size[0]))
+                        item.set_image(_icon.IconWidget(ICON_ENGINE, size[0]))
                     item.connect("activate", self.__im_menu_item_activate_cb, engine)
                     submenu.add(item)
 
-        item = gtk.ImageMenuItem(_("No input method"))
-        item.set_image (_icon.IconWidget("gtk-close", size[0]))
+        item = gtk.ImageMenuItem(_("Turn off input method"))
+        item.set_image(_icon.IconWidget("gtk-close", size[0]))
         item.connect("activate", self.__im_menu_item_activate_cb, None)
-        menu.add (item)
+        menu.add(item)
 
         menu.show_all()
         menu.set_take_focus(False)
@@ -395,8 +398,14 @@ class Panel(ibus.PanelBase):
 
     def __status_icon_activate_cb(self, status_icon):
         if not self.__focus_ic:
-            return
-        menu = self.__create_im_menu()
+            menu = gtk.Menu()
+            item = gtk.ImageMenuItem(_("No input window"))
+            size = gtk.icon_size_lookup(gtk.ICON_SIZE_MENU)
+            item.set_image(_icon.IconWidget("gtk-info", size[0]))
+            menu.add(item)
+            menu.show_all()
+        else:
+            menu = self.__create_im_menu()
         menu.popup(None, None,
                 gtk.status_icon_position_menu,
                 0,
@@ -424,6 +433,7 @@ class Panel(ibus.PanelBase):
             about_dialog.set_documenters(["Huang Peng <shawn.p.huang@gmail.com>"])
             about_dialog.set_translator_credits(_("translator-credits"))
             about_dialog.set_logo_icon_name("ibus")
+            about_dialog.set_icon_name("ibus")
             about_dialog.run()
             about_dialog.destroy()
         elif command == gtk.STOCK_QUIT: 
@@ -450,4 +460,3 @@ class Panel(ibus.PanelBase):
             self.__setup_pid = 0
         self.__setup_pid = os.spawnl(os.P_NOWAIT, self.__setup_cmd, "ibus-setup")
 
-gobject.type_register(Panel, "IBusPanel")
