@@ -28,6 +28,7 @@ import icon
 from menu import *
 
 class ToolButton(gtk.ToolButton, PropItem):
+    __gtype_name__ = "IBusToolButton"
     __gsignals__ = {
         "property-activate" : (
             gobject.SIGNAL_RUN_FIRST,
@@ -36,9 +37,9 @@ class ToolButton(gtk.ToolButton, PropItem):
         }
 
     def __init__(self, prop):
-        gtk.ToolButton.__init__ (self, label = prop.label.text)
+        gtk.ToolButton.__init__(self, label = prop.label.text)
         self.set_homogeneous(False)
-        PropItem.__init__ (self, prop)
+        PropItem.__init__(self, prop)
         self.property_changed()
 
     def set_icon_name(self, icon_name):
@@ -82,6 +83,7 @@ class ToolButton(gtk.ToolButton, PropItem):
 
 
 class ToggleToolButton(gtk.ToggleToolButton, PropItem):
+    __gtype_name__ = "IBusToggleToolButton"
     __gsignals__ = {
         "property-activate" : (
             gobject.SIGNAL_RUN_FIRST,
@@ -89,10 +91,10 @@ class ToggleToolButton(gtk.ToggleToolButton, PropItem):
             (gobject.TYPE_STRING, gobject.TYPE_INT)),
         }
 
-    def __init__ (self, prop):
-        gtk.ToggleToolButton.__init__ (self)
+    def __init__(self, prop):
+        gtk.ToggleToolButton.__init__(self)
         self.set_homogeneous(False)
-        PropItem.__init__ (self, prop)
+        PropItem.__init__(self, prop)
         self.property_changed()
 
     def set_icon_name(self, icon_name):
@@ -139,12 +141,14 @@ class ToggleToolButton(gtk.ToggleToolButton, PropItem):
         self.emit("property-activate", self._prop.key, self._prop.state)
 
 class SeparatorToolItem(gtk.SeparatorToolItem, PropItem):
+    __gtype_name__ = "IBusSeparatorToolItem"
     def __init__ (self, prop):
         gtk.SeparatorToolItem.__init__(self)
         self.set_homogeneous(False)
         PropItem.__init__(self, prop)
 
 class MenuToolButton(ToggleToolButton):
+    __gtype_name__ = "IBusMenuToolButton"
     # __gsignals__ = {
     #        "property-activate" : (
     #            gobject.SIGNAL_RUN_FIRST,
@@ -152,7 +156,7 @@ class MenuToolButton(ToggleToolButton):
     #            (gobject.TYPE_STRING, gobject.TYPE_INT)),
     #        }
 
-    def __init__ (self, prop):
+    def __init__(self, prop):
         super(MenuToolButton, self).__init__(prop)
         self._menu = Menu(prop)
         self._menu.connect("deactivate", lambda m: self.set_active(False))
@@ -166,7 +170,7 @@ class MenuToolButton(ToggleToolButton):
         if self.get_active():
             self._menu.popup(0, gtk.get_current_event_time(), self)
 
-gobject.type_register(ToolButton, "ToolButton")
-gobject.type_register(ToggleToolButton, "IBusToggleToolButton")
-gobject.type_register(MenuToolButton, "MenuToolButton")
-
+    def destroy(self):
+        self._menu.destroy()
+        self._menu = None
+        super(MenuToolButton, self).destroy()
